"""
Example for using CGShading and CGFunction. This code is directly translated
from procedural C code and is definitely not good Python style.
"""
import array
import math
import random
import sys

import Cocoa
import objc
import Quartz
from objc import super

# Global variables
frequency = [0.0, 0.0, 0.0, 0.0]
startPoint = Quartz.CGPoint(0.0, 0.0)
startRadius = 0.0
startExtend = False
endPoint = Quartz.CGPoint(0.0, 0.0)
endRadius = 0.0
endExtend = False

shading = None
function = None
getFunction = None
getShading = None
colorspace = None

DEFAULT_WIDTH = 256
DEFAULT_HEIGHT = 256

MAX_WIDTH = 1000
MAX_HEIGHT = 1000


def randomPoint():
    return Quartz.CGPoint(random.random(), random.random())


def evaluate1(components, input_value, output_value):
    out = []
    for k in range(components - 1):
        out.append(1 + (math.sin(input[0] * frequency[k])) / 2)
    out.append(1)
    return out


def getFunction1(colorspace):
    if sys.maxsize > 2 ** 32:
        a_type = "d"
    else:
        a_type = "f"
    domain = array.array(a_type, [-2 * math.pi, 2 * math.pi])
    function_range = array.array(a_type, [0, 1, 0, 1, 0, 1, 0, 1, 0, 1])
    components = 1 + Quartz.CGColorSpaceGetNumberOfComponents(colorspace)

    return Quartz.CGFunctionCreate(
        components, 1, domain, components, function_range, evaluate1
    )


def evaluate2(components, input_value, output_value):
    c = [0.510, 0.188, 0.910, 0.122]

    v = input_value[0]
    out = []
    for k in range(components - 1):
        if v < 0.5:
            out.append(c[k] * 2 * (0.5 - v))
        else:
            out.append(c[k] * 2 * (v - 0.5))
    out.append(1)
    return out


def getFunction2(colorspace):
    domain = [0, 1]
    function_range = [0, 1, 0, 1, 0, 1, 0, 1, 0, 1]

    components = 1 + Quartz.CGColorSpaceGetNumberOfComponents(colorspace)
    return Quartz.CGFunctionCreate(
        components, 1, domain, components, function_range, evaluate2
    )


def evaluate3(components, input_value, output_value):
    c = [0.3, 0, 0, 0]

    out = []
    for k in range(components - 1):
        out.append(c[k] * input_value[0])
    out.append(1)
    return out


def getFunction3(colorspace):
    domain = [0, 1]
    function_range = [0, 1, 0, 1, 0, 1, 0, 1, 0, 1]

    components = 1 + Quartz.CGColorSpaceGetNumberOfComponents(colorspace)
    return Quartz.CGFunctionCreate(
        components, 1, domain, components, function_range, evaluate3
    )


def getAxialShading(colorspace, function):
    return Quartz.CGShadingCreateAxial(
        colorspace, startPoint, endPoint, function, startExtend, endExtend
    )


def getRadialShading(colorspace, function):
    return Quartz.CGShadingCreateRadial(
        colorspace,
        startPoint,
        startRadius,
        endPoint,
        endRadius,
        function,
        startExtend,
        endExtend,
    )


class MyQuartzView(Cocoa.NSView):
    def initWithFrame_(self, frameRect):
        global startPoint, startRadius, startExtend
        global endPoint, endRadius, endExtend

        super(MyQuartzView, self).initWithFrame_(frameRect)

        startPoint = Quartz.CGPoint(0, 0)
        startRadius = 0
        startExtend = False

        endPoint = Quartz.CGPointMake(0, 0)
        endRadius = 0
        endExtend = False

        return self

    def drawRect_(self, rect):
        currentContext = Cocoa.NSGraphicsContext.currentContext().graphicsPort()

        # Note that at this point the current context CTM is set up such
        # that the context size corresponds to the size of the view
        # i.e. one unit in the context == one pixel
        # Also, the origin is in the bottom left of the view with +y pointing up
        global getFunction

        bounds = self.bounds()

        angle = 0
        sx = sy = 1
        width = bounds.size.width
        height = bounds.size.height

        if getFunction is None:
            self.randomize_(self)

        m = Quartz.CGAffineTransformIdentity
        m = Quartz.CGAffineTransformRotate(m, angle)
        m = Quartz.CGAffineTransformScale(m, width, height)
        m = Quartz.CGAffineTransformScale(m, sx, sy)

        Quartz.CGContextBeginPage(currentContext, bounds)

        Quartz.CGContextTranslateCTM(
            currentContext, bounds.size.width / 2, bounds.size.height / 2
        )
        Quartz.CGContextConcatCTM(currentContext, m)
        Quartz.CGContextTranslateCTM(currentContext, -0.5, -0.5)

        Quartz.CGContextSaveGState(currentContext)

        Quartz.CGContextClipToRect(currentContext, Quartz.CGRectMake(0, 0, 1, 1))
        Quartz.CGContextSetRGBFillColor(currentContext, 0.7, 0.7, 0.9, 1)
        Quartz.CGContextFillRect(currentContext, Quartz.CGRectMake(0, 0, 1, 1))

        Quartz.CGContextDrawShading(currentContext, shading)

        Quartz.CGContextRestoreGState(currentContext)

        Quartz.CGContextSaveGState(currentContext)
        Quartz.CGContextClipToRect(currentContext, Quartz.CGRectMake(0, 0, 1, 1))
        Quartz.CGContextSetRGBStrokeColor(currentContext, 1, 0, 0, 1)

        if getShading == getRadialShading:
            Quartz.CGContextAddArc(
                currentContext,
                startPoint.x,
                startPoint.y,
                startRadius,
                math.radians(0),
                math.radians(360),
                True,
            )
            Quartz.CGContextClosePath(currentContext)
            Quartz.CGContextMoveToPoint(
                currentContext, endPoint.x + endRadius, endPoint.y
            )
            Quartz.CGContextAddArc(
                currentContext,
                endPoint.x,
                endPoint.y,
                endRadius,
                math.radians(0),
                math.radians(360),
                True,
            )
            Quartz.CGContextClosePath(currentContext)

        Quartz.CGContextMoveToPoint(currentContext, startPoint.x + 0.01, startPoint.y)
        Quartz.CGContextAddArc(
            currentContext,
            startPoint.x,
            startPoint.y,
            0.01,
            math.radians(0),
            math.radians(360),
            True,
        )
        Quartz.CGContextClosePath(currentContext)
        Quartz.CGContextMoveToPoint(currentContext, startPoint.x, startPoint.y)
        Quartz.CGContextAddLineToPoint(currentContext, endPoint.x, endPoint.y)

        ctm = Quartz.CGContextGetCTM(currentContext)
        Quartz.CGContextConcatCTM(currentContext, Quartz.CGAffineTransformInvert(ctm))
        Quartz.CGContextStrokePath(currentContext)
        Quartz.CGContextRestoreGState(currentContext)

        Quartz.CGContextSaveGState(currentContext)
        Quartz.CGContextSetGrayStrokeColor(currentContext, 0, 1)
        Quartz.CGContextAddRect(currentContext, Quartz.CGRectMake(0, 0, 1, 1))
        ctm = Quartz.CGContextGetCTM(currentContext)
        Quartz.CGContextConcatCTM(currentContext, Quartz.CGAffineTransformInvert(ctm))
        Quartz.CGContextStrokePath(currentContext)
        Quartz.CGContextRestoreGState(currentContext)

        Quartz.CGContextEndPage(currentContext)

        Quartz.CGContextFlush(currentContext)

    @objc.IBAction
    def randomize_(self, sender):
        global colorspace, getFunction, getShading
        global function, shading
        global startPoint, startRadius, endPoint, endRadius

        if colorspace is None:
            colorspace = Quartz.CGColorSpaceCreateDeviceRGB()

        for k in range(len(frequency)):
            frequency[k] = random.random()

        startPoint = randomPoint()
        startRadius = random.random() / 2
        endPoint = randomPoint()
        endRadius = random.random() / 2

        if getFunction == getFunction1:
            getFunction = getFunction2

        elif getFunction == getFunction2:
            getFunction = getFunction3

        else:
            getFunction = getFunction1

        if getShading == getAxialShading:
            getShading = getRadialShading
        else:
            getShading = getAxialShading

        function = getFunction(colorspace)
        shading = getShading(colorspace, function)

        self.setNeedsDisplay_(True)

    @objc.IBAction
    def toggleStartExtend_(self, sender):
        global startExtend, shading

        startExtend = not startExtend
        shading = getShading(colorspace, function)

        self.setNeedsDisplay_(True)

    @objc.IBAction
    def toggleEndExtend_(self, sender):
        global endExtend, shading

        endExtend = not endExtend
        shading = getShading(colorspace, function)

        self.setNeedsDisplay_(True)



import AppDrawing
import DrawingBasics
import LaunchServices
import objc
import Quartz
import Utilities

# import Carbon.QT


BEST_BYTE_ALIGNMENT = 16


def COMPUTE_BEST_BYTES_PER_ROW(bpr):
    return (int(bpr) + BEST_BYTE_ALIGNMENT - 1) & ~(BEST_BYTE_ALIGNMENT - 1)


# We need to ensure that the raster data stays alive until we clean up
# the context, store it here.
_rasterDataForContext = {}


def createRGBBitmapContext(
    width, height, wantDisplayColorSpace, needsTransparentBitmap
):
    # This routine allocates data for a pixel array that contains width*height
    # pixels where each pixel is 4 bytes. The format is 8-bit ARGB or XRGB, depending on
    # whether needsTransparentBitmap is true. In order to get the recommended
    # pixel alignment, the bytesPerRow is rounded up to the nearest multiple
    # of BEST_BYTE_ALIGNMENT bytes.

    # Minimum bytes per row is 4 bytes per sample * number of samples.
    bytesPerRow = width * 4
    # Round to nearest multiple of BEST_BYTE_ALIGNMENT.
    bytesPerRow = COMPUTE_BEST_BYTES_PER_ROW(bytesPerRow)

    # Allocate the data for the raster. The total amount of data is bytesPerRow
    # times the number of rows. The function 'calloc' is used so that the
    # memory is initialized to 0.
    try:
        rasterData = objc.allocateBuffer(int(bytesPerRow * height))
    except MemoryError:
        return None

    # The wantDisplayColorSpace argument passed to the function determines
    # whether or not to use the display color space or the generic calibrated
    # RGB color space. The needsTransparentBitmap argument determines whether
    # create a context that records alpha or not.
    if wantDisplayColorSpace:
        cs = Utilities.getTheDisplayColorSpace()
    else:
        cs = Utilities.getTheCalibratedRGBColorSpace()

    if needsTransparentBitmap:
        transparency = Quartz.kCGImageAlphaPremultipliedFirst
    else:
        transparency = Quartz.kCGImageAlphaPremultipliedFirst

    context = Quartz.CGBitmapContextCreate(
        rasterData, width, height, 8, bytesPerRow, cs, transparency
    )
    if context is None:
        return None

    _rasterDataForContext[context] = rasterData

    # Either clear the rect or paint with opaque white, depending on
    # the needs of the caller.
    if needsTransparentBitmap:
        # Clear the context bits so they are transparent.
        Quartz.CGContextClearRect(context, Quartz.CGRectMake(0, 0, width, height))

    else:
        # Since the drawing destination is opaque, first paint
        # the context bits to white.
        Quartz.CGContextSaveGState(context)
        Quartz.CGContextSetFillColorWithColor(
            context, Utilities.getRGBOpaqueWhiteColor()
        )
        Quartz.CGContextFillRect(context, Quartz.CGRectMake(0, 0, width, height))
        Quartz.CGContextRestoreGState(context)

    return context


def myCGContextGetBitmapInfo(c):
    if hasattr(Quartz, "CGBitmapContextGetBitmapInfo"):
        return Quartz.CGBitmapContextGetBitmapInfo(c)
    else:
        return Quartz.CGBitmapContextGetAlphaInfo(c)


# createImageFromBitmapContext creates a CGImageRef
# from a bitmap context. Calling this routine
# transfers 'ownership' of the raster data
# in the bitmap context, to the image. If the
# image can't be created, this routine frees
# the memory associated with the raster.
def createImageFromBitmapContext(c):
    rasterData = _rasterDataForContext[c]
    # We own the data, hence remove from the mapping
    del _rasterDataForContext[c]

    imageDataSize = Quartz.CGBitmapContextGetBytesPerRow(
        c
    ) * Quartz.CGBitmapContextGetHeight(c)

    if rasterData is None:
        print("Context is not a bitmap context!")

    # Create the data provider from the image data
    dataProvider = Quartz.CGDataProviderCreateWithData(
        None, rasterData, imageDataSize, None
    )
    if dataProvider is None:
        print("Couldn't create data provider!")
        return None

    # Now create the image. The parameters for the image closely match
    # the parameters of the bitmap context. This code uses a NULL
    # decode array and shouldInterpolate is true.
    image = Quartz.CGImageCreate(
        Quartz.CGBitmapContextGetWidth(c),
        Quartz.CGBitmapContextGetHeight(c),
        Quartz.CGBitmapContextGetBitsPerComponent(c),
        Quartz.CGBitmapContextGetBitsPerPixel(c),
        Quartz.CGBitmapContextGetBytesPerRow(c),
        Quartz.CGBitmapContextGetColorSpace(c),
        myCGContextGetBitmapInfo(c),
        dataProvider,
        None,
        True,
        Quartz.kCGRenderingIntentDefault,
    )

    if image is None:
        print("Couldn't create image!")
        return None
    return image


def exportCGImageToFileWithQT(image, url, outputFormat, dpi):
    """
    Export an image using QuickTime API's.

    This function can't possibly work, as the relevant APIs aren't available
    through MacPython. The code below is mostly there in case someone fixes
    the MacPython QuickTime bindings.
    """
    # XXX: Finish
    return

    if outputFormat.lower() == LaunchServices.kUTTypeTIFF.lower():
        imageExportType = LaunchServices.kQTFileTypeTIFF

    elif outputFormat.lower() == LaunchServices.kUTTypePNG.lower():
        imageExportType = LaunchServices.kQTFileTypePNG

    elif outputFormat.lower() == LaunchServices.kUTTypeJPEG.lower():
        imageExportType = LaunchServices.kQTFileTypeJPEG

    else:
        print("Requested image export format %@s unsupported" % (outputFormat,))
        return

        result, dataRef, dataRefType = Quartz.QTNewDataReferenceFromCFURL(
            url, 0, None, None
        )
        if result == 0:
            result, graphicsExporter = Quartz.OpenADefaultComponent(
                Quartz.GraphicsExporterComponentType,
                imageExportType,
                graphicsExporter,  # noqa: F821
            )
            if result == 0:
                result = Quartz.GraphicsExportSetInputCGImage(graphicsExporter, image)
                if result == 0:
                    result = Quartz.GraphicsExportSetResolution(
                        graphicsExporter,
                        Quartz.FloatToFixed(dpi),
                        Quartz.FloatToFixed(dpi),
                    )
                if result == 0:
                    result = Quartz.GraphicsExportSetOutputDataReference(
                        graphicsExporter, dataRef, dataRefType
                    )
                if result == 0:
                    result, sizeWritten = Quartz.GraphicsExportDoExport(
                        graphicsExporter, None
                    )
                Quartz.CloseComponent(graphicsExporter)

        if dataRef:
            Quartz.DisposeHandle(dataRef)

        if result:
            print("QT export got bad result = %d!" % (result,))


def exportCGImageToFileWithDestination(image, url, outputFormat, dpi):
    # Create an image destination at the supplied URL that
    # corresponds to the output image format. The destination will
    # only contain 1 image.
    imageDestination = Quartz.CGImageDestinationCreateWithURL(
        url, outputFormat, 1, None
    )

    if imageDestination is None:
        print("Couldn't create image destination!")
        return

    # Create an options dictionary with the X&Y resolution of the image
    options = {
        Quartz.kCGImagePropertyDPIWidth: dpi,
        Quartz.kCGImagePropertyDPIHeight: dpi,
    }

    # Add the image with the options dictionary to the destination.
    Quartz.CGImageDestinationAddImage(imageDestination, image, options)

    # When all the images are added to the destination, finalize it.
    Quartz.CGImageDestinationFinalize(imageDestination)


def MakeImageDocument(url, imageType, exportInfo):
    # First make a bitmap context for a US Letter size
    # raster at the requested resolution.
    dpi = exportInfo.dpi
    width = int(8.5 * dpi)
    height = int(11 * dpi)

    # For JPEG output type the bitmap should not be transparent. If other types are added that
    # do not support transparency, this code should be updated to check for those types as well.
    needTransparentBitmap = imageType.lower() != LaunchServices.kUTTypeJPEG.lower()

    # Create an RGB Bitmap context using the generic calibrated RGB color space
    # instead of the display color space.
    useDisplayColorSpace = False
    c = createRGBBitmapContext(
        width, height, useDisplayColorSpace, needTransparentBitmap
    )

    if c is None:
        print("Couldn't make destination bitmap context")
        return -1

    # Scale the coordinate system based on the resolution in dots per inch.
    Quartz.CGContextScaleCTM(c, dpi / 72, dpi / 72)

    # Set the font smoothing parameter to false since it's better to
    # draw any text without special LCD text rendering when creating
    # rendered data for export.
    if hasattr(Quartz, "CGContextSetShouldSmoothFonts"):
        Quartz.CGContextSetShouldSmoothFonts(c, False)

    # Set the scaling factor for shadows. This is a hack so that
    # drawing code that needs to know the scaling factor can
    # obtain it. Better would be that DispatchDrawing and the code
    # it calls would take this scaling factor as a parameter.
    Utilities.setScalingFactor(dpi / 72)

    # Draw into that raster...
    AppDrawing.DispatchDrawing(c, exportInfo.command)

    # Set the scaling factor back to 1.0.
    Utilities.setScalingFactor(1.0)

    # Create an image from the raster data. Calling
    # createImageFromBitmapContext gives up ownership
    # of the raster data used by the context.
    image = createImageFromBitmapContext(c)

    # Release the context now that the image is created.
    del c

    if image is None:
        # Users of this code should update this to be an error code they find useful.
        return -1

    # Now export the image.
    if exportInfo.useQTForExport:
        exportCGImageToFileWithQT(image, url, imageType, exportInfo.dpi)
    else:
        exportCGImageToFileWithDestination(image, url, imageType, exportInfo.dpi)


def MakeTIFFDocument(url, exportInfo):
    return MakeImageDocument(url, LaunchServices.kUTTypeTIFF, exportInfo)


def MakePNGDocument(url, exportInfo):
    return MakeImageDocument(url, LaunchServices.kUTTypePNG, exportInfo)


def MakeJPEGDocument(url, exportInfo):
    return MakeImageDocument(url, LaunchServices.kUTTypeJPEG, exportInfo)


def createCGLayerForDrawing(c):
    rect = Quartz.CGRectMake(0, 0, 50, 50)

    # Make the layer the size of the rectangle that
    # this code draws into the layer.
    layerSize = rect.size

    # Create the layer to draw into.
    layer = Quartz.CGLayerCreateWithContext(c, layerSize, None)
    if layer is None:
        return None

    # Get the context corresponding to the layer.
    layerContext = Quartz.CGLayerGetContext(layer)
    if layerContext is None:
        return None

    # $ Set the fill color to opaque black.
    Quartz.CGContextSetFillColorWithColor(
        layerContext, Utilities.getRGBOpaqueBlackColor()
    )

    # Draw the content into the layer.
    Quartz.CGContextFillRect(layerContext, rect)

    # Now the layer has the contents needed.
    return layer


def doSimpleCGLayer(context):
    # Create the layer.
    layer = createCGLayerForDrawing(context)

    if layer is None:
        print("Couldn't create layer!")
        return

    # Get the size of the layer created.
    s = Quartz.CGLayerGetSize(layer)

    # Clip to a rect that corresponds to
    # a grid of 8x8 layer objects.
    Quartz.CGContextClipToRect(
        context, Quartz.CGRectMake(0, 0, 8 * s.width, 8 * s.height)
    )

    # Paint 8 rows of layer objects.
    for j in range(8):
        Quartz.CGContextSaveGState(context)
        # Paint 4 columns of layer objects, moving
        # across the drawing canvas by skipping a
        # square on the grid each time across.
        for _ in range(4):
            # Draw the layer at the current origin.
            Quartz.CGContextDrawLayerAtPoint(context, Quartz.CGPointZero, layer)
            # Translate across two layer widths.
            Quartz.CGContextTranslateCTM(context, 2 * s.width, 0)

        Quartz.CGContextRestoreGState(context)
        # Translate to the left one layer width on
        # even loop counts and to the right one
        # layer width on odd loop counts. Each
        # time through the outer loop, translate up
        # one layer height.
        if j % 2:
            Quartz.CGContextTranslateCTM(context, s.width, s.height)
        else:
            Quartz.CGContextTranslateCTM(context, -s.width, s.height)


def createAlphaOnlyContext(width, height):
    # This routine allocates data for a pixel array that contains
    # width*height pixels, each pixel is 1 byte. The format is
    # 8 bits per pixel, where the data is the alpha value of the pixel.

    # Minimum bytes per row is 1 byte per sample * number of samples.
    bytesPerRow = width
    # Round to nearest multiple of BEST_BYTE_ALIGNMENT.
    bytesPerRow = COMPUTE_BEST_BYTES_PER_ROW(bytesPerRow)

    # Allocate the data for the raster. The total amount of data is bytesPerRow
    # // times the number of rows. The function 'calloc' is used so that the
    # // memory is initialized to 0.
    try:
        rasterData = objc.allocateBuffer(bytesPerRow * height)
    except MemoryError:
        return None

    # This type of context is only available in Panther and later, otherwise
    # this fails and returns a NULL context. The color space for an alpha
    # // only context is NULL and the BitmapInfo value is kCGImageAlphaOnly.
    context = Quartz.CGBitmapContextCreate(
        rasterData, width, height, 8, bytesPerRow, None, Quartz.kCGImageAlphaOnly
    )
    if context is None:
        print("Couldn't create the context!")
        return None

    _rasterDataForContext[context] = rasterData

    # Clear the context bits so they are initially transparent.
    Quartz.CGContextClearRect(context, Quartz.CGRectMake(0, 0, width, height))

    return context


# createMaskFromAlphaOnlyContext creates a CGImageRef
# from an alpha-only bitmap context. Calling this routine
# transfers 'ownership' of the raster data in the bitmap
# context, to the image. If the image can't be created, this
# routine frees the memory associated with the raster.


def createMaskFromAlphaOnlyContext(alphaContext):
    rasterData = _rasterDataForContext[alphaContext]
    # We own the data, hence remove from the mapping
    del _rasterDataForContext[alphaContext]

    imageDataSize = Quartz.CGBitmapContextGetBytesPerRow(
        alphaContext
    ) * Quartz.CGBitmapContextGetHeight(alphaContext)
    invertDecode = [1.0, 0.0]

    # Create the data provider from the image data.
    dataProvider = Quartz.CGDataProviderCreateWithData(
        None, rasterData, imageDataSize, None
    )

    if dataProvider is None:
        print("Couldn't create data provider!")
        return None

    mask = Quartz.CGImageMaskCreate(
        Quartz.CGBitmapContextGetWidth(alphaContext),
        Quartz.CGBitmapContextGetHeight(alphaContext),
        Quartz.CGBitmapContextGetBitsPerComponent(alphaContext),
        Quartz.CGBitmapContextGetBitsPerPixel(alphaContext),
        Quartz.CGBitmapContextGetBytesPerRow(alphaContext),
        dataProvider,
        # The decode is an inverted decode since a mask has the opposite
        # sense than alpha, i.e. 0 in a mask paints 100% and 1 in a mask
        # paints nothing.
        invertDecode,
        True,
    )

    if mask is None:
        print("Couldn't create image mask!")
        return None

    return mask


def doAlphaOnlyContext(context):
    # This code is going to capture the alpha coverage
    # of the drawing done by the doAlphaRects routine.
    # The value passed here as the width and height is
    # the size of the bounding rectangle of that drawing.
    width = 520
    height = 400
    alphaContext = createAlphaOnlyContext(width, height)
    if context is None:
        print("Couldn't create the alpha-only context!")
        return

    # Draw the content to the alpha-only context, capturing
    # the alpha coverage. The doAlphaRects routine paints
    # a series of translucent red rectangles.
    DrawingBasics.doAlphaRects(alphaContext)

    # Finished drawing to the context and now the raster contains
    # the alpha data captured from the drawing. Create
    # the mask from the data in the context.
    mask = createMaskFromAlphaOnlyContext(alphaContext)
    # This code is now finshed with the context so it can
    # release it.
    del alphaContext

    if mask is None:
        return

    # Set the fill color space.
    Quartz.CGContextSetFillColorSpace(
        context, Utilities.getTheCalibratedRGBColorSpace()
    )
    opaqueBlue = (0.11, 0.208, 0.451, 1.0)
    # Set the painting color to opaque blue.
    Quartz.CGContextSetFillColor(context, opaqueBlue)
    # Draw the mask, painting the mask with blue. This colorizes
    # the image to blue and it is as if we painted the
    # alpha rects with blue instead of red.
    Quartz.CGContextDrawImage(context, Quartz.CGRectMake(0, 0, width, height), mask)


_pdfDoc = None
_pdfURL = None
_width = 0
_height = 0


def getThePDFDoc(url):
    """
    This function caches a CGPDFDocumentRef for
    the most recently requested PDF document.
    """
    global _pdfDoc, _pdfURL, _width, _height

    if url is None:
        return None, 0, 0

    # See whether to update the cached PDF document.
    if _pdfDoc is None or url != _pdfURL:
        # Release any cached document or URL.
        _pdfDoc = Quartz.CGPDFDocumentCreateWithURL(url)
        if _pdfDoc is not None:
            pdfMediaRect = Quartz.CGPDFDocumentGetMediaBox(_pdfDoc, 1)
            _width = pdfMediaRect.size.width
            _height = pdfMediaRect.size.height
            # Keep the URL of the PDF file being cached.
            _pdfURL = url

        else:
            _pdfURL = None

    if _pdfDoc is not None:
        return _pdfDoc, _width, _height

    else:
        return None, 0, 0


# Defining this scales the content down by 1/3.
DOSCALING = True


def TilePDFNoBuffer(context, url):
    # The amount of area to tile should really be based on the
    # window/document. Here it is hard coded to a US Letter
    # size document. This may draw too many or too few tiles
    # for the area actually being filled.
    fillwidth = 612.0
    fillheight = 792.0
    extraOffset = 6.0
    pdfDoc, tileX, tileY = getThePDFDoc(url)
    if pdfDoc is None:
        print("Couldn't get the PDF document!")
        return

    if DOSCALING:
        # Make the tiles 1/3 the size of the PDF document.
        tileX /= 3
        tileY /= 3
        extraOffset /= 3

    # Space the tiles by the tile width and height
    # plus extraOffset units in each dimension.
    tileOffsetX = extraOffset + tileX
    tileOffsetY = extraOffset + tileY

    # Tile the PDF document.
    for h in range(0, int(fillheight), int(tileOffsetY)):
        for w in range(0, int(fillwidth), int(tileOffsetX)):
            Quartz.CGContextDrawPDFDocument(
                context, Quartz.CGRectMake(w, h, tileX, tileY), pdfDoc, 1
            )


def TilePDFWithOffscreenBitmap(context, url):
    # Again this should really be computed based on
    # the area intended to be tiled.
    fillwidth = 612.0
    fillheight = 792.0
    extraOffset = 6.0

    pdfDoc, tileX, tileY = getThePDFDoc(url)
    if pdfDoc is None:
        print("Couldn't get the PDF document")
        return

    if DOSCALING:
        # Make the tiles 1/3 the size of the PDF document.
        tileX /= 3
        tileY /= 3
        extraOffset /= 3

    # Space the tiles by the tile width and height
    # plus extraOffset units in each dimension.
    tileOffsetX = extraOffset + tileX
    tileOffsetY = extraOffset + tileY

    # Since the bitmap context is for use with the display
    # and should capture alpha, these are the values
    # to pass to createRGBBitmapContext.
    useDisplayColorSpace = True
    needTransparentBitmap = True
    bitmapContext = createRGBBitmapContext(
        tileX, tileY, useDisplayColorSpace, needTransparentBitmap
    )
    if bitmapContext is None:
        print("Couldn't create bitmap context!")
        return

    # Draw the PDF document one time into the bitmap context.
    Quartz.CGContextDrawPDFDocument(
        bitmapContext, Quartz.CGRectMake(0, 0, tileX, tileY), pdfDoc, 1
    )

    # Create an image from the raster data. Calling
    # createImageFromBitmapContext gives up ownership
    # of the raster data used by the context.
    image = createImageFromBitmapContext(bitmapContext)

    # Release the context now that the image is created.
    del bitmapContext

    if image is None:
        return

    # Now tile the image.
    for h in range(0, int(fillheight), int(tileOffsetY)):
        for w in range(0, int(fillwidth), int(tileOffsetX)):
            Quartz.CGContextDrawImage(
                context, Quartz.CGRectMake(w, h, tileX, tileY), image
            )


def createLayerWithImageForContext(c, url):
    layerSize = Quartz.CGSize()
    pdfDoc, layerSize.width, layerSize.height = getThePDFDoc(url)
    if pdfDoc is None:
        return None

    if DOSCALING:
        # Make the layer 1/3 the size of the PDF document.
        layerSize.width /= 3
        layerSize.height /= 3

    # Create the layer to draw into.
    layer = Quartz.CGLayerCreateWithContext(c, layerSize, None)
    if layer is None:
        return None

    # Get the context corresponding to the layer. Note
    # that this is a 'Get' function so the code must
    # not release the context.
    layerContext = Quartz.CGLayerGetContext(layer)
    if layerContext is None:
        return None

    # Draw the PDF document into the layer.
    Quartz.CGContextDrawPDFDocument(
        layerContext,
        Quartz.CGRectMake(0, 0, layerSize.width, layerSize.height),
        pdfDoc,
        1,
    )

    # Now the layer has the contents needed.
    return layer


def TilePDFWithCGLayer(context, url):
    # Again this should really be computed based on
    # the area intended to be tiled.
    fillwidth = 612.0
    fillheight = 792.0
    layer = createLayerWithImageForContext(context, url)
    if layer is None:
        print("Couldn't create the layer!")
        return

    # Compute the tile size and offset.
    s = Quartz.CGLayerGetSize(layer)
    tileX = s.width
    tileY = s.height

    if DOSCALING:
        # Space the tiles by the tile width and height
        # plus an extra 2 units in each dimension.
        tileOffsetX = 2.0 + tileX
        tileOffsetY = 2.0 + tileY
    else:
        # Add 6 units to the offset in each direction
        # if there is no scaling of the source PDF document.
        tileOffsetX = 6.0 + tileX
        tileOffsetY = 6.0 + tileY

    # Now draw the contents of the layer to the context.
    # The layer is drawn at its true size (the size of
    # the tile) with its origin located at the corner
    # of each tile.
    for h in range(0, int(fillheight), int(tileOffsetY)):
        for w in range(0, int(fillwidth), int(tileOffsetX)):
            Quartz.CGContextDrawLayerAtPoint(context, Quartz.CGPointMake(w, h), layer)



import BitmapContext
import Cocoa
import objc
import PDFHandling
import Utilities

# Initial defaults
_dpi = 144
_useQT = False


def getURLToExport(suffix):
    savePanel = Cocoa.NSSavePanel.savePanel()

    initialFileName = "BasicDrawing.%s" % (suffix,)

    if (
        savePanel.runModalForDirectory_file_(None, initialFileName)
        == Cocoa.NSFileHandlingPanelOKButton
    ):
        return savePanel.URL()

    return None


class MyAppController(Cocoa.NSObject):
    theView = objc.IBOutlet()
    currentDPIMenuItem = objc.IBOutlet()
    currentExportStyleMenuItem = objc.IBOutlet()

    @objc.IBAction
    def print_(self, sender):
        self.theView.print_(sender)

    def updateDPIMenu_(self, sender):
        if self.currentDPIMenuItem is not sender:
            # Uncheck the previous item.
            if self.currentDPIMenuItem is not None:
                self.currentDPIMenuItem.setState_(Cocoa.NSOffState)
            # Update to the current item.
            self.currentDPIMenuItem = sender
            # Check new menu item.
            self.currentDPIMenuItem.setState_(Cocoa.NSOnState)

    def updateExportStyleMenu_(self, sender):
        if self.currentExportStyleMenuItem is not sender:
            # Uncheck the previous item.
            if self.currentExportStyleMenuItem is not None:
                self.currentExportStyleMenuItem.setState_(Cocoa.NSOffState)
            # Update to the current item.
            self.currentExportStyleMenuItem = sender
            # Check new menu item.
            self.currentExportStyleMenuItem.setState_(Cocoa.NSOnState)

    @objc.IBAction
    def setExportResolution_(self, sender):
        global _dpi
        _dpi = sender.tag()
        self.updateDPIMenu_(sender)

    @objc.IBAction
    def setUseQT_(self, sender):
        global _useQT
        _useQT = True
        self.updateExportStyleMenu_(sender)

    @objc.IBAction
    def setUseCGImageSource_(self, sender):
        global _useQT
        _useQT = False
        self.updateExportStyleMenu_(sender)

    def setupExportInfo_(self, exportInfoP):
        # Use the printable version of the current command. This produces
        # the best results for exporting.
        exportInfoP.command = self.theView.currentPrintableCommand()
        exportInfoP.fileType = "    "  # unused
        exportInfoP.useQTForExport = _useQT
        exportInfoP.dpi = _dpi

    @objc.IBAction
    def exportAsPDF_(self, sender):
        url = getURLToExport("pdf")
        if url is not None:
            exportInfo = Utilities.ExportInfo()
            self.setupExportInfo_(exportInfo)
            PDFHandling.MakePDFDocument(url, exportInfo)

    @objc.IBAction
    def exportAsPNG_(self, sender):
        url = getURLToExport("png")
        if url is not None:
            exportInfo = Utilities.ExportInfo()
            self.setupExportInfo_(exportInfo)
            BitmapContext.MakePNGDocument(url, exportInfo)

    @objc.IBAction
    def exportAsTIFF_(self, sender):
        url = getURLToExport("tif")
        if url is not None:
            exportInfo = Utilities.ExportInfo()
            self.setupExportInfo_(exportInfo)
            BitmapContext.MakeTIFFDocument(url, exportInfo)

    @objc.IBAction
    def exportAsJPEG_(self, sender):
        url = getURLToExport("jpg")
        if url is not None:
            exportInfo = Utilities.ExportInfo()
            self.setupExportInfo_(exportInfo)
            BitmapContext.MakeJPEGDocument(url, exportInfo)

    def validateMenuItem_(self, menuItem):
        if menuItem.tag == _dpi:
            self.currentDPIMenuItem = menuItem
            menuItem.setState_(True)
        elif menuItem.action() == "setUseQT:":
            if _useQT:
                self.currentDPIMenuItem = menuItem
                menuItem.setState_(True)
            else:
                menuItem.setState_(False)

        elif menuItem.action() == "setUseCGImageSource:":
            if _useQT:
                self.currentDPIMenuItem = menuItem
                menuItem.setState_(True)
            else:
                menuItem.setState_(False)

        return True



# Make sure all code is loaded
import AppDrawing  # noqa: F401
import BitmapContext  # noqa: F401
import ColorAndGState  # noqa: F401
import CoordinateSystem  # noqa: F401
import DataProvidersAndConsumers  # noqa: F401
import DrawingBasics  # noqa: F401
import EPSPrinting  # noqa: F401
import FrameworkTextDrawing  # noqa: F401
import FrameworkUtilities  # noqa: F401
import ImageMasking  # noqa: F401
import Images  # noqa: F401
import MyAppController  # noqa: F401
import MyView  # noqa: F401
import PathDrawing  # noqa: F401
import PatternDrawing  # noqa: F401
import PDFHandling  # noqa: F401
import QuartzTextDrawing  # noqa: F401
import Shadings  # noqa: F401
import ShadowsAndTransparencyLayers  # noqa: F401
import Utilities  # noqa: F401
from PyObjCTools import AppHelper

AppHelper.runEventLoop()



import array

import Quartz
import Utilities


def doColorSpaceFillAndStroke(context):
    theColorSpace = Utilities.getTheCalibratedRGBColorSpace()
    opaqueRed = (0.663, 0.0, 0.031, 1.0)  # red,green,blue,alpha
    aBlue = (0.482, 0.62, 0.871, 1.0)  # red,green,blue,alpha

    # Set the fill color space to be the generic calibrated RGB color space.
    Quartz.CGContextSetFillColorSpace(context, theColorSpace)
    # Set the fill color to opaque red. The number of elements in the
    # array passed to this function must be the number of color
    # components in the current fill color space plus 1 for alpha.
    Quartz.CGContextSetFillColor(context, opaqueRed)

    # Set the stroke color space to be the generic calibrated RGB color space.
    Quartz.CGContextSetStrokeColorSpace(context, theColorSpace)
    # Set the stroke color to opaque blue. The number of elements
    # in the array passed to this function must be the number of color
    # components in the current stroke color space plus 1 for alpha.
    Quartz.CGContextSetStrokeColor(context, aBlue)

    Quartz.CGContextSetLineWidth(context, 8.0)
    # Rectangle 1.
    Quartz.CGContextBeginPath(context)
    Quartz.CGContextAddRect(context, Quartz.CGRectMake(20.0, 20.0, 100.0, 100.0))
    Quartz.CGContextDrawPath(context, Quartz.kCGPathFillStroke)

    # Continue to use the stroke colorspace already set
    # but change the stroke alpha value to a semitransparent blue.
    aBlue = list(aBlue)
    aBlue[3] = 0.5
    Quartz.CGContextSetStrokeColor(context, aBlue)
    # Rectangle 2.
    Quartz.CGContextBeginPath(context)
    Quartz.CGContextAddRect(context, Quartz.CGRectMake(140.0, 20.0, 100.0, 100.0))
    Quartz.CGContextDrawPath(context, Quartz.kCGPathFillStroke)

    # Don't release the color space since this routine
    # didn't create it.


_opaqueRedColor = None
_opaqueBlueColor = None
_transparentBlueColor = None


def drawWithColorRefs(context):
    global _opaqueRedColor
    global _opaqueBlueColor
    global _transparentBlueColor

    # Initialize the CGColorRefs if necessary
    if _opaqueRedColor is None:
        # Initialize the color array to an opaque red
        # in the generic calibrated RGB color space.
        color = (0.663, 0.0, 0.031, 1.0)
        theColorSpace = Utilities.getTheCalibratedRGBColorSpace()
        # Create a CGColorRef for opaque red.
        _opaqueRedColor = Quartz.CGColorCreate(theColorSpace, color)
        # Make the color array correspond to an opaque blue color.
        color = (0.482, 0.62, 0.87, 1.0)
        # Create another Quartz.CGColorRef for opaque blue.
        _opaqueBlueColor = Quartz.CGColorCreate(theColorSpace, color)
        # Create a new CGColorRef from the opaqueBlue CGColorRef
        # but with a different alpha value.
        _transparentBlueColor = Quartz.CGColorCreateCopyWithAlpha(_opaqueBlueColor, 0.5)
        if (
            _opaqueRedColor is None
            or _opaqueBlueColor is None
            or _transparentBlueColor is None
        ):
            print("Couldn't create one of the CGColorRefs!!!")
            return

    # Set the fill color to the opaque red CGColor object.
    Quartz.CGContextSetFillColorWithColor(context, _opaqueRedColor)
    # Set the stroke color to the opaque blue CGColor object.
    Quartz.CGContextSetStrokeColorWithColor(context, _opaqueBlueColor)

    Quartz.CGContextSetLineWidth(context, 8.0)
    # Draw the first rectangle.
    Quartz.CGContextBeginPath(context)
    Quartz.CGContextAddRect(context, Quartz.CGRectMake(20.0, 20.0, 100.0, 100.0))
    Quartz.CGContextDrawPath(context, Quartz.kCGPathFillStroke)

    # Set the stroke color to be that of the transparent blue
    # CGColor object.
    Quartz.CGContextSetStrokeColorWithColor(context, _transparentBlueColor)
    # Draw a second rectangle to the right of the first one.
    Quartz.CGContextBeginPath(context)
    Quartz.CGContextAddRect(context, Quartz.CGRectMake(140.0, 20.0, 100.0, 100.0))
    Quartz.CGContextDrawPath(context, Quartz.kCGPathFillStroke)


def doIndexedColorDrawGraphics(context):
    theBaseRGBSpace = Utilities.getTheCalibratedRGBColorSpace()
    lookupTable = array.array("B", (0,) * 6)
    opaqueRed = (0, 1)  # index, alpha
    aBlue = (1, 1)  # index, alpha

    # Set the first 3 values in the lookup table to a red of
    # 169/255 = 0.663, no green, and blue = 8/255 = 0.031. This makes
    # the first entry in the lookup table a shade of red.
    lookupTable[0] = 169
    lookupTable[1] = 0
    lookupTable[2] = 8

    # Set the second 3 values in the lookup table to a red value
    # of 123/255 = 0.482, a green value of 158/255 = 0.62, and
    # a blue value of 222/255 = 0.871. This makes the second entry
    # in the lookup table a shade of blue.
    lookupTable[3] = 123
    lookupTable[4] = 158
    lookupTable[5] = 222

    # Create the indexed color space with this color lookup table,
    # using the RGB color space as the base color space and a 2 element
    # color lookup table to characterize the indexed color space.
    theIndexedSpace = Quartz.CGColorSpaceCreateIndexed(theBaseRGBSpace, 1, lookupTable)
    if theIndexedSpace is not None:
        Quartz.CGContextSetStrokeColorSpace(context, theIndexedSpace)
        Quartz.CGContextSetFillColorSpace(context, theIndexedSpace)

        # Set the stroke color to an opaque blue.
        Quartz.CGContextSetStrokeColor(context, aBlue)
        # Set the fill color to an opaque red.
        Quartz.CGContextSetFillColor(context, opaqueRed)

        Quartz.CGContextSetLineWidth(context, 8.0)
        # Draw the first rectangle.
        Quartz.CGContextBeginPath(context)
        Quartz.CGContextAddRect(context, Quartz.CGRectMake(20.0, 20.0, 100.0, 100.0))
        Quartz.CGContextDrawPath(context, Quartz.kCGPathFillStroke)

        # Continue to use the stroke colorspace already set
        # but change the stroke alpha value to a semitransparent value
        # while leaving the index value unchanged.
        aBlue = list(aBlue)
        aBlue[1] = 0.5
        Quartz.CGContextSetStrokeColor(context, aBlue)
        # Draw another rectangle to the right of the first one.
        Quartz.CGContextBeginPath(context)
        Quartz.CGContextAddRect(context, Quartz.CGRectMake(140.0, 20.0, 100.0, 100.0))
        Quartz.CGContextDrawPath(context, Quartz.kCGPathFillStroke)
    else:
        print("Couldn't make the indexed color space!")


def drawWithGlobalAlpha(context):
    rect = Quartz.CGRectMake(40.0, 210.0, 100.0, 100.0)
    color = [1.0, 0.0, 0.0, 1.0]  # opaque red
    # Set the fill color space to that returned by getTheCalibratedRGBColorSpace.
    Quartz.CGContextSetFillColorSpace(
        context, Utilities.getTheCalibratedRGBColorSpace()
    )

    Quartz.CGContextSetFillColor(context, color)
    for _ in range(2):
        Quartz.CGContextSaveGState(context)
        # Paint the leftmost rect on this row with 100% opaque red.
        Quartz.CGContextFillRect(context, rect)

        Quartz.CGContextTranslateCTM(context, rect.size.width + 70.0, 0.0)
        # Set the alpha value of this rgba color to 0.5.
        color[3] = 0.5
        # Use the new color as the fill color in the graphics state.
        Quartz.CGContextSetFillColor(context, color)
        # Paint the center rect on this row with 50% opaque red.
        Quartz.CGContextFillRect(context, rect)

        Quartz.CGContextTranslateCTM(context, rect.size.width + 70.0, 0.0)
        # Set the alpha value of this rgba color to 0.25.
        color[3] = 0.25
        # Use the new color as the fill color in the graphics state.
        Quartz.CGContextSetFillColor(context, color)
        # Paint the rightmost rect on this row with 25% opaque red.
        Quartz.CGContextFillRect(context, rect)
        Quartz.CGContextRestoreGState(context)
        # After restoring the graphics state, the fill color is set to
        # that prior to calling CGContextSaveGState, that is, opaque
        # red. The coordinate system is also restored.

        # Now set the context global alpha value to 50% opaque.
        Quartz.CGContextSetAlpha(context, 0.5)
        # Translate down for a second row of rectangles.
        Quartz.CGContextTranslateCTM(context, 0.0, -(rect.size.height + 70.0))
        # Reset the alpha value of the color array to fully opaque.
        color[3] = 1.0


def drawWithColorBlendMode(context, url):
    # A pleasant green color.
    green = [0.584, 0.871, 0.318, 1.0]

    # Create a CGPDFDocument object from the URL.
    pdfDoc = Quartz.CGPDFDocumentCreateWithURL(url)
    if pdfDoc is None:
        print("Couldn't create CGPDFDocument from URL!")
        return

    # Obtain the media box for page 1 of the PDF document.
    pdfRect = Quartz.CGPDFDocumentGetMediaBox(pdfDoc, 1)
    # Set the origin of the rectangle to (0,0).
    pdfRect.origin.x = pdfRect.origin.y = 0

    # Graphic 1, the left portion of the figure.
    Quartz.CGContextTranslateCTM(context, 20, 10 + Quartz.CGRectGetHeight(pdfRect) / 2)

    # Draw the PDF document.
    Quartz.CGContextDrawPDFDocument(context, pdfRect, pdfDoc, 1)

    # Set the fill color space to that returned by getTheCalibratedRGBColorSpace.
    Quartz.CGContextSetFillColorSpace(
        context, Utilities.getTheCalibratedRGBColorSpace()
    )
    # Set the fill color to green.
    Quartz.CGContextSetFillColor(context, green)

    # Graphic 2, the top-right portion of the figure.
    Quartz.CGContextTranslateCTM(
        context,
        Quartz.CGRectGetWidth(pdfRect) + 10,
        Quartz.CGRectGetHeight(pdfRect) / 2 + 10,
    )

    # Draw the PDF document again.
    Quartz.CGContextDrawPDFDocument(context, pdfRect, pdfDoc, 1)

    # Make a fill rectangle that is the same size as the PDF document
    # but inset each side by 80 units in x and 20 units in y.
    insetRect = Quartz.CGRectInset(pdfRect, 80, 20)
    # Fill the rectangle with green. Because the fill color is opaque and
    # the blend mode is Normal, this obscures the drawing underneath.
    Quartz.CGContextFillRect(context, insetRect)

    # Graphic 3, the bottom-right portion of the figure.
    Quartz.CGContextTranslateCTM(context, 0, -(10 + Quartz.CGRectGetHeight(pdfRect)))

    # Draw the PDF document again.
    Quartz.CGContextDrawPDFDocument(context, pdfRect, pdfDoc, 1)

    # Set the blend mode to kCGBlendModeColor which will
    # colorize the destination with subsequent drawing.
    Quartz.CGContextSetBlendMode(context, Quartz.kCGBlendModeColor)
    # Draw the rectangle on top of the PDF document. The portion of the
    # background that is covered by the rectangle is colorized
    # with the fill color.
    Quartz.CGContextFillRect(context, insetRect)


def createEllipsePath(context, center, ellipseSize):
    Quartz.CGContextSaveGState(context)
    # Translate the coordinate origin to the center point.
    Quartz.CGContextTranslateCTM(context, center.x, center.y)
    # Scale the coordinate system to half the width and height
    # of the ellipse.
    Quartz.CGContextScaleCTM(context, ellipseSize.width / 2, ellipseSize.height / 2)
    Quartz.CGContextBeginPath(context)
    # Add a circular arc to the path, centered at the origin and
    # with a radius of 1.0. This radius, together with the
    # scaling above for the width and height, produces an ellipse
    # of the correct size.
    Quartz.CGContextAddArc(
        context, 0.0, 0.0, 1.0, 0.0, Utilities.DEGREES_TO_RADIANS(360.0), 0.0
    )
    # Close the path so that this path is suitable for stroking,
    # should that be desired.
    Quartz.CGContextClosePath(context)
    Quartz.CGContextRestoreGState(context)


_opaqueBrownColor = None
_opaqueOrangeColor = None


def doClippedEllipse(context):
    global _opaqueBrownColor, _opaqueOrangeColor

    theCenterPoint = Quartz.CGPoint(120.0, 120.0)
    theEllipseSize = Quartz.CGSize(100.0, 200.0)
    dash = [2.0]

    # Initialize the CGColorRefs if necessary.
    if _opaqueBrownColor is None:
        # The initial value of the color array is an
        # opaque brown in an RGB color space.
        color = [0.325, 0.208, 0.157, 1.0]
        theColorSpace = Utilities.getTheCalibratedRGBColorSpace()
        # Create a CGColorRef for opaque brown.
        _opaqueBrownColor = Quartz.CGColorCreate(theColorSpace, color)
        # Make the color array correspond to an opaque orange.
        color = [0.965, 0.584, 0.059, 1.0]
        # Create another CGColorRef for opaque orange.
        _opaqueOrangeColor = Quartz.CGColorCreate(theColorSpace, color)

    # Draw two ellipses centered about the same point, one
    # rotated 45 degrees from the other.
    Quartz.CGContextSaveGState(context)
    # Ellipse 1
    createEllipsePath(context, theCenterPoint, theEllipseSize)
    Quartz.CGContextSetFillColorWithColor(context, _opaqueBrownColor)
    Quartz.CGContextFillPath(context)
    # Translate and rotate about the center point of the ellipse.
    Quartz.CGContextTranslateCTM(context, theCenterPoint.x, theCenterPoint.y)
    # Rotate by 45 degrees.
    Quartz.CGContextRotateCTM(context, Utilities.DEGREES_TO_RADIANS(45))
    # Ellipse 2
    # CGPointZero is a pre-defined Quartz point corresponding to
    # the coordinate (0,0).
    createEllipsePath(context, Quartz.CGPointZero, theEllipseSize)
    Quartz.CGContextSetFillColorWithColor(context, _opaqueOrangeColor)
    Quartz.CGContextFillPath(context)
    Quartz.CGContextRestoreGState(context)

    Quartz.CGContextTranslateCTM(context, 170.0, 0.0)
    # Now use the first ellipse as a clipping area prior to
    # painting the second ellipse.
    Quartz.CGContextSaveGState(context)
    # Ellipse 3
    createEllipsePath(context, theCenterPoint, theEllipseSize)
    Quartz.CGContextSetStrokeColorWithColor(context, _opaqueBrownColor)
    Quartz.CGContextSetLineDash(context, 0, dash, 1)
    # Stroke the path with a dash.
    Quartz.CGContextStrokePath(context)
    # Ellipse 4
    createEllipsePath(context, theCenterPoint, theEllipseSize)
    # Clip to the elliptical path.
    Quartz.CGContextClip(context)
    Quartz.CGContextTranslateCTM(context, theCenterPoint.x, theCenterPoint.y)
    # Rotate by 45 degrees.
    Quartz.CGContextRotateCTM(context, Utilities.DEGREES_TO_RADIANS(45))
    # Ellipse 5
    createEllipsePath(context, Quartz.CGPointZero, theEllipseSize)
    Quartz.CGContextSetFillColorWithColor(context, _opaqueOrangeColor)
    Quartz.CGContextFillPath(context)
    Quartz.CGContextRestoreGState(context)



import sys

import Cocoa

# import DataProvidersAndConsumers
import Images
import Quartz
import Utilities


def exportImageWithMaskFromURLWithDestination(
    context,
    imageURL,
    imagewidth,
    imageheight,
    bitsPerComponent,
    theMaskingImageURL,
    maskwidth,
    maskheight,
):

    imageBitsPerPixel = bitsPerComponent * 3
    bytesPerRow = ((imagewidth * imageBitsPerPixel) + 7) / 8
    shouldInterpolate = True
    imageDataProvider = Quartz.CGDataProviderCreateWithURL(imageURL)
    if imageDataProvider is None:
        print("Couldn't create Image Data provider!")
        return

    colorspace = Utilities.getTheCalibratedRGBColorSpace()
    image = Quartz.CGImageCreate(
        imagewidth,
        imageheight,
        bitsPerComponent,
        imageBitsPerPixel,
        bytesPerRow,
        colorspace,
        Quartz.kCGImageAlphaNone,
        imageDataProvider,
        None,
        shouldInterpolate,
        Quartz.kCGRenderingIntentDefault,
    )
    del imageDataProvider
    if image is None:
        print("Couldn't create CGImageRef for this data!")
        return

    imageRect = Quartz.CGRectMake(0.0, imageheight, imagewidth, imageheight)
    # Draw the image.
    Quartz.CGContextDrawImage(context, imageRect, image)

    # Now the mask.
    maskDataProvider = Quartz.CGDataProviderCreateWithURL(theMaskingImageURL)
    if maskDataProvider is None:
        print("Couldn't create Image Data provider!")
        return

    mask = Quartz.CGImageMaskCreate(
        maskwidth,
        maskheight,
        bitsPerComponent,
        bitsPerComponent,
        maskwidth,
        maskDataProvider,
        None,
        shouldInterpolate,
    )
    del maskDataProvider
    if mask is None:
        print("Couldn't create CGImageRef for mask data!")
        return

    # Draw the mask below the image.
    maskRect = Quartz.CGRectMake(0.0, 0.0, maskwidth, maskheight)
    Quartz.CGContextDrawImage(context, maskRect, mask)

    # Create a new CGImage object, the image, masked with mask.
    imageMaskedWithImage = Quartz.CGImageCreateWithMask(image, mask)
    # Once the new image is created, we can release the image
    # and the mask which make it up. Quartz retains what it needs
    # for the new masked image.
    del image
    del mask

    if imageMaskedWithImage is None:
        print("Couldn't create image masked with mask!")
        return

    imageRect = Quartz.CGRectMake(imagewidth, imageheight / 2, imagewidth, imageheight)
    # Draw the masked image to the right of the image and its mask.
    Quartz.CGContextDrawImage(context, imageRect, imageMaskedWithImage)

    # Of course this is a total hack.
    outPath = b"/tmp/imageout.png"
    exportURL = Cocoa.CFURLCreateFromFileSystemRepresentation(
        None, outPath, len(outPath), False
    )

    if exportURL is not None:
        Images.exportCGImageToPNGFileWithDestination(imageMaskedWithImage, exportURL)


if sys.version_info[0] == 2:

    def make_bytes(values):
        return "".join(map(chr, values))


else:

    def make_bytes(values):
        return bytes(values)


_data = make_bytes(
    (
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0x7F,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFE,
        0x1F,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xF8,
        0x00,
        0x03,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xF0,
        0x00,
        0x00,
        0xF8,
        0xE7,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xE0,
        0x00,
        0x00,
        0x00,
        0x40,
        0x7F,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xC0,
        0x00,
        0x00,
        0x00,
        0x00,
        0x7F,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFE,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x1F,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFC,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x03,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xF8,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x01,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xF8,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x01,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xE0,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xE0,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x3F,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xC0,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x3F,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0x80,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x0F,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x0F,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x07,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x06,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0xFF,
        0xFF,
        0xFF,
        0xFE,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0xFF,
        0xFF,
        0xFF,
        0xFE,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x7F,
        0xFF,
        0xFF,
        0xFE,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x7F,
        0xFF,
        0xFF,
        0xFE,
        0x00,
        0x00,
        0x00,
        0x00,
        0x01,
        0xC0,
        0x00,
        0x00,
        0x7F,
        0xFF,
        0xFF,
        0xFE,
        0x00,
        0x00,
        0x00,
        0x00,
        0x0F,
        0xF8,
        0x00,
        0x00,
        0x7F,
        0xFF,
        0xFF,
        0xFE,
        0x00,
        0x00,
        0x00,
        0x0F,
        0xFF,
        0xF8,
        0x00,
        0x00,
        0x7F,
        0xFF,
        0xFF,
        0xFE,
        0x00,
        0x00,
        0x00,
        0x1F,
        0xFF,
        0xFC,
        0x00,
        0x00,
        0x7F,
        0xFF,
        0xFF,
        0xFE,
        0x00,
        0x00,
        0x00,
        0x7F,
        0xFF,
        0xFC,
        0x00,
        0x00,
        0x7F,
        0xFF,
        0xFF,
        0xF8,
        0x00,
        0x00,
        0x00,
        0xFF,
        0xFF,
        0xFC,
        0x00,
        0x00,
        0xFF,
        0xFF,
        0xFF,
        0xF8,
        0x00,
        0x00,
        0x03,
        0xFF,
        0xFF,
        0xFF,
        0x00,
        0x00,
        0xFF,
        0xFF,
        0xFF,
        0xF0,
        0x00,
        0x00,
        0x03,
        0xFF,
        0xFF,
        0xFF,
        0x00,
        0x00,
        0x7F,
        0xFF,
        0xFF,
        0xF8,
        0x00,
        0x00,
        0x0F,
        0xFF,
        0xFF,
        0xFF,
        0x00,
        0x00,
        0x7F,
        0xFF,
        0xFF,
        0xF8,
        0x00,
        0x00,
        0x0F,
        0xFF,
        0xFF,
        0xFF,
        0x00,
        0x00,
        0xFF,
        0xFF,
        0xFF,
        0xF0,
        0x00,
        0x00,
        0x1F,
        0xFF,
        0xFF,
        0xFF,
        0x00,
        0x00,
        0xFF,
        0xFF,
        0xFF,
        0xE0,
        0x00,
        0x00,
        0x3F,
        0xFF,
        0xFF,
        0xFF,
        0x80,
        0x01,
        0xFF,
        0xFF,
        0xFF,
        0xE0,
        0x00,
        0x00,
        0x7F,
        0xFF,
        0xFF,
        0xFF,
        0x80,
        0x00,
        0x1F,
        0xFF,
        0xFF,
        0xE0,
        0x00,
        0x00,
        0x7F,
        0xFF,
        0xFF,
        0xFF,
        0x80,
        0x00,
        0x1F,
        0xFF,
        0xFF,
        0xE0,
        0x00,
        0x00,
        0x3F,
        0xFF,
        0xFF,
        0xFF,
        0xC0,
        0x00,
        0x1F,
        0xFF,
        0xFF,
        0xF8,
        0x00,
        0x00,
        0x3F,
        0xFF,
        0xFF,
        0xFF,
        0xE0,
        0x00,
        0x1F,
        0xFF,
        0xFF,
        0xF8,
        0x00,
        0x00,
        0x7F,
        0xFF,
        0xFF,
        0xFF,
        0xC0,
        0x00,
        0x1F,
        0xFF,
        0xFF,
        0xFC,
        0x00,
        0x00,
        0x7F,
        0xFF,
        0xFF,
        0xFE,
        0x40,
        0x00,
        0x0F,
        0xFF,
        0xFF,
        0xFC,
        0x00,
        0x00,
        0xFF,
        0xFF,
        0xFF,
        0xFE,
        0x00,
        0x00,
        0x1F,
        0xFF,
        0xFF,
        0xFC,
        0x00,
        0x00,
        0xFF,
        0xFF,
        0xFF,
        0xC0,
        0x00,
        0x00,
        0x1F,
        0xFF,
        0xFF,
        0xFC,
        0x00,
        0x01,
        0xFF,
        0xFF,
        0xFF,
        0x00,
        0x00,
        0x00,
        0x3F,
        0xFF,
        0xFF,
        0xFC,
        0x00,
        0x05,
        0xEF,
        0xFF,
        0xFE,
        0x00,
        0x00,
        0x00,
        0x3F,
        0xFF,
        0xFF,
        0xF0,
        0x00,
        0x3F,
        0x00,
        0x03,
        0xFC,
        0x00,
        0x00,
        0x00,
        0x3F,
        0xFF,
        0xFF,
        0xE0,
        0x00,
        0x7C,
        0x00,
        0x00,
        0x78,
        0x1F,
        0x00,
        0x00,
        0x7F,
        0xFF,
        0xFF,
        0xC0,
        0x00,
        0x38,
        0x00,
        0x00,
        0x78,
        0x3C,
        0x00,
        0x01,
        0xFF,
        0xFF,
        0xFF,
        0xC0,
        0x00,
        0x78,
        0x00,
        0x00,
        0x70,
        0x18,
        0x00,
        0x01,
        0xFF,
        0xFF,
        0xFF,
        0xF0,
        0x00,
        0x78,
        0x1F,
        0x00,
        0x30,
        0x00,
        0x00,
        0x01,
        0xFF,
        0xFF,
        0xFF,
        0xFE,
        0x00,
        0x7C,
        0x3F,
        0x00,
        0x18,
        0x00,
        0x00,
        0x01,
        0xFF,
        0xFF,
        0xFF,
        0xFE,
        0x00,
        0x00,
        0x00,
        0x00,
        0x38,
        0x00,
        0x00,
        0x03,
        0xFF,
        0xFF,
        0xFF,
        0xFE,
        0x00,
        0x00,
        0x80,
        0x00,
        0x3C,
        0x00,
        0x0C,
        0x03,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0x00,
        0x00,
        0x00,
        0x00,
        0x3C,
        0x20,
        0x1C,
        0x03,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0x00,
        0x00,
        0x04,
        0x00,
        0x3C,
        0x00,
        0x3C,
        0x03,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0x00,
        0x70,
        0xBF,
        0x86,
        0x3C,
        0x1F,
        0xFC,
        0x0B,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xA0,
        0x11,
        0xF0,
        0x0E,
        0x3C,
        0x1F,
        0xFE,
        0x8B,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xA0,
        0x19,
        0xF0,
        0x0C,
        0x3C,
        0x0F,
        0xFF,
        0x0B,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xB0,
        0x1D,
        0xFE,
        0x1C,
        0x7E,
        0x0F,
        0xFF,
        0x03,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xB8,
        0x1C,
        0xFF,
        0x3C,
        0xFE,
        0x03,
        0xFE,
        0x03,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFC,
        0x1E,
        0x7F,
        0xF8,
        0xDE,
        0x00,
        0x7C,
        0x03,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFE,
        0x1E,
        0x7F,
        0xF1,
        0xDF,
        0x30,
        0x03,
        0x83,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFE,
        0x1F,
        0x3F,
        0xE3,
        0x9F,
        0x10,
        0x3F,
        0x83,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFE,
        0x0F,
        0xFF,
        0x83,
        0xDF,
        0x80,
        0x1F,
        0x83,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFE,
        0x03,
        0xFC,
        0x03,
        0xDF,
        0x81,
        0x8F,
        0x83,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFE,
        0x07,
        0xFE,
        0x1F,
        0x8F,
        0x00,
        0x07,
        0x83,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0x07,
        0xFE,
        0x3C,
        0x06,
        0x00,
        0x01,
        0x83,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0x03,
        0xFC,
        0x7C,
        0x00,
        0x00,
        0x01,
        0x83,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0x01,
        0xF8,
        0x7F,
        0x00,
        0x00,
        0x01,
        0x83,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0x01,
        0xF8,
        0xFF,
        0xE0,
        0x30,
        0x01,
        0x83,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0x00,
        0xF1,
        0xEF,
        0xF9,
        0xE0,
        0x03,
        0x83,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0x80,
        0xF1,
        0xFF,
        0xFF,
        0x80,
        0x0F,
        0x83,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFC,
        0x03,
        0xE2,
        0xFF,
        0xFE,
        0x00,
        0x1F,
        0x87,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0x83,
        0xF0,
        0x00,
        0x00,
        0x1C,
        0x3F,
        0x87,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xC3,
        0xF0,
        0x00,
        0x01,
        0xF8,
        0x0F,
        0x87,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xC3,
        0xF0,
        0x03,
        0xFF,
        0xF0,
        0x5F,
        0x07,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xC1,
        0xFF,
        0xC7,
        0xFF,
        0xE0,
        0x7F,
        0x0F,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xE1,
        0xFF,
        0xF1,
        0xFF,
        0x80,
        0x2F,
        0x0F,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xE1,
        0xFF,
        0xF8,
        0x0F,
        0xC0,
        0x06,
        0x1F,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xF4,
        0xFF,
        0xFE,
        0x0F,
        0xF8,
        0x44,
        0x1F,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xF4,
        0xFF,
        0xFF,
        0xFF,
        0xF8,
        0x64,
        0x3F,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xF9,
        0xFF,
        0xFF,
        0xFF,
        0x3C,
        0xE4,
        0x7F,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFD,
        0x9F,
        0xFF,
        0xFC,
        0x1F,
        0xC0,
        0x7F,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFD,
        0x1F,
        0xFF,
        0xFC,
        0x03,
        0xC0,
        0x7F,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFE,
        0x01,
        0xFF,
        0xFF,
        0xFF,
        0xC0,
        0x7F,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFE,
        0x00,
        0xFF,
        0xFF,
        0xFF,
        0x00,
        0x7F,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFE,
        0x00,
        0xFF,
        0xFF,
        0xFF,
        0x00,
        0x7F,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFE,
        0x80,
        0x7F,
        0xFF,
        0xFF,
        0x00,
        0x3F,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFE,
        0x80,
        0x1F,
        0xFF,
        0xFF,
        0x00,
        0x3F,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFE,
        0xC0,
        0x0F,
        0xFF,
        0xFF,
        0x00,
        0x1F,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xE0,
        0x07,
        0xFF,
        0xFF,
        0x00,
        0x0F,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xF0,
        0x03,
        0xFF,
        0xFF,
        0x00,
        0x1F,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0x70,
        0x01,
        0xFF,
        0xFC,
        0x00,
        0x17,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFC,
        0x78,
        0x00,
        0x7F,
        0xF0,
        0x00,
        0x07,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xF0,
        0xFC,
        0x00,
        0x00,
        0x00,
        0x00,
        0x07,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0x03,
        0xFE,
        0x00,
        0x00,
        0x00,
        0x00,
        0x07,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFE,
        0x1F,
        0xFF,
        0x80,
        0x00,
        0x00,
        0x00,
        0x07,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFC,
        0x7F,
        0x7F,
        0xC0,
        0x00,
        0x00,
        0x00,
        0x07,
        0xFF,
        0xFF,
    )
)


def getMaskData1():
    return _data


_data2 = make_bytes(
    (
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0x03,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFC,
        0x00,
        0x1F,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xE0,
        0x00,
        0x01,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0x80,
        0x00,
        0x00,
        0x3F,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0x00,
        0x00,
        0x00,
        0x1F,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFC,
        0x00,
        0x00,
        0x00,
        0x07,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xF8,
        0x00,
        0x00,
        0x00,
        0x03,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xF0,
        0x00,
        0x00,
        0x00,
        0x01,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xE0,
        0x00,
        0x00,
        0x00,
        0x01,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xC0,
        0x00,
        0x00,
        0x00,
        0x00,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0x80,
        0x00,
        0x00,
        0x00,
        0x00,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x7F,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFE,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x3F,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFE,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x3F,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFC,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x1F,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xF8,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x1F,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xF8,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x0F,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xF0,
        0x00,
        0x00,
        0x00,
        0x01,
        0x80,
        0x07,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xF0,
        0x00,
        0x00,
        0x00,
        0x03,
        0xC0,
        0x07,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xF0,
        0x00,
        0x00,
        0x00,
        0x0B,
        0xE0,
        0x07,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xE0,
        0x00,
        0x00,
        0x00,
        0x07,
        0xF0,
        0x03,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xE0,
        0x00,
        0x00,
        0x00,
        0x1F,
        0xF4,
        0x83,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xC0,
        0x00,
        0x00,
        0x00,
        0x3F,
        0xE4,
        0x03,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xC0,
        0x00,
        0x00,
        0x00,
        0x3F,
        0xE4,
        0x43,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0x80,
        0x00,
        0x00,
        0x00,
        0x3F,
        0xE4,
        0x4B,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0x80,
        0x00,
        0x00,
        0x02,
        0xFF,
        0xE4,
        0x5B,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0x00,
        0x00,
        0x00,
        0x02,
        0xFF,
        0xE0,
        0x5B,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0x00,
        0x00,
        0x07,
        0xC1,
        0xFF,
        0xE0,
        0x59,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0x18,
        0x00,
        0x7F,
        0xF0,
        0xFE,
        0x00,
        0x79,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFE,
        0x18,
        0x00,
        0x78,
        0x0F,
        0xFE,
        0x04,
        0xE1,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFE,
        0x18,
        0x00,
        0xB0,
        0x47,
        0xFF,
        0xFF,
        0xE1,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFE,
        0x10,
        0x00,
        0xC4,
        0x69,
        0xFF,
        0xFF,
        0xC3,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFE,
        0x10,
        0x01,
        0xFF,
        0xE1,
        0xFC,
        0x07,
        0xC1,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFC,
        0x00,
        0x01,
        0xFF,
        0xF8,
        0x78,
        0x01,
        0xC5,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFC,
        0x04,
        0x01,
        0xFF,
        0xF0,
        0x78,
        0x01,
        0xC5,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFC,
        0x0C,
        0x00,
        0xFF,
        0xF8,
        0x7E,
        0x3F,
        0xC1,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFC,
        0x0C,
        0x00,
        0x7F,
        0xF0,
        0x18,
        0xFF,
        0xC3,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFC,
        0x07,
        0x00,
        0x7F,
        0xF4,
        0x1F,
        0xFF,
        0xE3,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFC,
        0x23,
        0x00,
        0x7F,
        0xE0,
        0x3E,
        0xFF,
        0xE3,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xF8,
        0x11,
        0x00,
        0x7F,
        0xEC,
        0x5F,
        0xBF,
        0xE3,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xF8,
        0x01,
        0x00,
        0x3F,
        0xCE,
        0x7E,
        0x3F,
        0xE3,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xF8,
        0x08,
        0x00,
        0x7F,
        0x80,
        0x2E,
        0x3F,
        0xE3,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xF8,
        0x06,
        0x00,
        0x7F,
        0x00,
        0x6E,
        0x3F,
        0xEB,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xF0,
        0x00,
        0x00,
        0x7E,
        0x0D,
        0xFE,
        0xFF,
        0xEB,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xF0,
        0x00,
        0x00,
        0x3C,
        0x00,
        0xFE,
        0x3F,
        0xEB,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xF0,
        0x00,
        0x00,
        0x50,
        0x00,
        0xFE,
        0x3F,
        0xCB,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xF0,
        0x00,
        0x00,
        0x01,
        0x98,
        0xFF,
        0x3F,
        0xCB,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xF0,
        0x00,
        0x00,
        0xC7,
        0xE1,
        0xFF,
        0x3F,
        0x8B,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xF0,
        0x00,
        0x00,
        0x40,
        0xFF,
        0xFF,
        0xBF,
        0x8B,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xF0,
        0x00,
        0x00,
        0xE0,
        0x1F,
        0xFF,
        0xDF,
        0x0B,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xF0,
        0x00,
        0x00,
        0xE8,
        0x63,
        0xFF,
        0xDF,
        0x0F,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xF0,
        0x00,
        0x02,
        0xFC,
        0xF9,
        0xFF,
        0xEF,
        0x0F,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xF0,
        0x00,
        0x03,
        0xFE,
        0x7F,
        0xF8,
        0x1E,
        0x0F,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xE0,
        0x00,
        0x03,
        0x7E,
        0x0F,
        0xF9,
        0xBE,
        0x05,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xE0,
        0x00,
        0x01,
        0x7F,
        0xC1,
        0xF3,
        0xFC,
        0x05,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xE0,
        0x00,
        0x01,
        0x3D,
        0xF8,
        0x0F,
        0x7C,
        0x05,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xE0,
        0x00,
        0x01,
        0xBC,
        0x7F,
        0xFF,
        0xF8,
        0x02,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xE0,
        0x00,
        0x00,
        0xBE,
        0xFF,
        0xFF,
        0xF8,
        0x02,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xC0,
        0x00,
        0x00,
        0x1D,
        0xFF,
        0xFF,
        0xF0,
        0x00,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xC0,
        0x00,
        0x00,
        0x0F,
        0xF7,
        0xFF,
        0xE0,
        0x00,
        0x7F,
        0xFF,
        0xFF,
        0xFF,
        0xC0,
        0x00,
        0x00,
        0x0F,
        0xF3,
        0xFF,
        0xE0,
        0x00,
        0x7F,
        0xFF,
        0xFF,
        0xFF,
        0x80,
        0x00,
        0x00,
        0x03,
        0xF3,
        0xFF,
        0xC0,
        0x00,
        0x7F,
        0xFF,
        0xFF,
        0xFF,
        0x80,
        0x00,
        0x00,
        0x01,
        0xF7,
        0xFF,
        0x80,
        0x00,
        0x3F,
        0xFF,
        0xFF,
        0xFF,
        0x00,
        0x00,
        0x00,
        0x00,
        0x3F,
        0xFF,
        0x00,
        0x00,
        0x3F,
        0xFF,
        0xFF,
        0xFF,
        0x00,
        0x00,
        0x00,
        0x00,
        0x1F,
        0xFF,
        0x00,
        0x20,
        0x3F,
        0xFF,
        0xFF,
        0xFE,
        0x00,
        0x00,
        0x00,
        0x00,
        0x1F,
        0xFF,
        0x00,
        0x10,
        0x3F,
        0xFF,
        0xFF,
        0xFE,
        0x00,
        0x00,
        0x00,
        0x00,
        0x1F,
        0xFF,
        0x00,
        0x10,
        0x3F,
        0xFF,
        0xFF,
        0xFC,
        0x00,
        0x00,
        0x02,
        0x00,
        0x0F,
        0xFF,
        0x00,
        0x00,
        0x1F,
        0xFF,
        0xFF,
        0xFC,
        0x00,
        0x00,
        0x04,
        0x00,
        0x1F,
        0xFE,
        0x00,
        0x00,
        0x1F,
        0xFF,
        0xFF,
        0xF8,
        0x00,
        0x00,
        0x07,
        0x81,
        0x7F,
        0xFE,
        0x00,
        0x00,
        0x1F,
        0xFF,
        0xFF,
        0xF8,
        0x00,
        0x00,
        0x03,
        0xDF,
        0xFF,
        0xFE,
        0x00,
        0x00,
        0x1F,
        0xFF,
        0xFF,
        0xF8,
        0x00,
        0x00,
        0x07,
        0xFF,
        0xFF,
        0xFE,
        0x00,
        0x00,
        0x1F,
        0xFF,
        0xFF,
        0xF0,
        0x00,
        0x00,
        0x07,
        0xFF,
        0xFF,
        0xFE,
        0x00,
        0x00,
        0x1F,
        0xFF,
        0xFF,
        0xF0,
        0x00,
        0x40,
        0x07,
        0xFF,
        0xFF,
        0xFE,
        0x00,
        0x00,
        0x0F,
        0xFF,
        0xFF,
        0xF0,
        0x00,
        0xC0,
        0x03,
        0xFF,
        0xFF,
        0xFE,
        0x00,
        0x00,
        0x0F,
        0xFF,
        0xFF,
        0xE0,
        0x00,
        0xE0,
        0x07,
        0xFF,
        0xFF,
        0xFE,
        0x81,
        0x00,
        0x07,
        0xFF,
        0xFF,
        0xC0,
        0x01,
        0xE0,
        0x07,
        0xFF,
        0xFF,
        0xFE,
        0x01,
        0x00,
        0x07,
        0xFF,
        0xFF,
        0x80,
        0x0F,
        0xF0,
        0x03,
        0xFF,
        0xFF,
        0xFE,
        0x83,
        0x80,
        0x03,
        0xFF,
        0xFF,
        0x00,
        0x1F,
        0xF0,
        0x13,
        0xFF,
        0xFF,
        0xFE,
        0x03,
        0xE0,
        0x01,
        0xFF,
        0xFC,
        0x03,
        0x3F,
        0xF0,
        0x21,
        0xFF,
        0xFF,
        0xFE,
        0x03,
        0xFC,
        0x00,
        0x3F,
        0xF0,
        0x3F,
        0x3F,
        0xF8,
        0x3B,
        0xFF,
        0xFF,
        0xFE,
        0x03,
        0xFE,
        0xC0,
        0x0F,
        0xE3,
        0xFB,
        0x7F,
        0xF8,
        0x3B,
        0xFF,
        0xFF,
        0xFF,
        0x07,
        0xFF,
        0xFF,
        0x07,
        0x9F,
        0xFB,
        0x7F,
        0xFC,
        0x79,
        0xFF,
        0xFF,
        0xFF,
        0x07,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0x7F,
        0xFC,
        0x39,
        0xFF,
        0xFF,
        0xFF,
        0x07,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0x7F,
        0xFE,
        0x3F,
        0xFF,
        0xFF,
        0xFE,
        0x07,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0x7F,
        0xFE,
        0x1F,
        0xFF,
        0xFF,
        0xFE,
        0x0F,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0x7F,
        0xFE,
        0x1F,
        0x7F,
        0xFF,
        0xFE,
        0x0F,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0x7F,
        0xFF,
        0x1F,
        0xFE,
        0xFF,
        0xFC,
        0x0F,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0x0F,
        0xFF,
        0xFF,
        0xFF,
        0x1F,
        0xFF,
        0xEF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0x87,
        0xFF,
        0xFF,
        0xFE,
        0x1F,
        0xFF,
        0xEF,
        0xFF,
        0xFF,
        0xFF,
        0xBF,
        0xFF,
        0x82,
        0xFF,
        0xFF,
        0xFC,
        0x3F,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xBF,
        0xFF,
        0x83,
        0xFF,
        0xFF,
        0xFC,
        0x3F,
        0xFF,
        0xBF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xC1,
        0xFF,
        0xFF,
        0xF8,
        0x7F,
        0xFF,
        0xBF,
        0xFF,
        0xFF,
        0xFF,
        0xBF,
        0xFF,
        0xE0,
        0xFF,
        0xFF,
        0xF0,
        0xFF,
        0xFF,
        0xBF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
    )
)


def getMaskData2():
    return _data2


def doOneBitMaskImages(context):
    bitsPerComponent = 1
    bitsPerPixel = 1
    width = 96
    height = 96
    bytesPerRow = 12

    imageDataSize = bytesPerRow * height
    shouldInterpolate = True
    lightBlue = [0.482, 0.62, 0.871, 1.0]
    black = [0.0, 0.0, 0.0, 1.0]
    darkRed = [0.663, 0.0, 0.031, 1.0]
    darkGreen = [0.404, 0.808, 0.239, 1.0]
    darkBlue = [0.11, 0.208, 0.451, 1.0]
    purple = [0.69, 0.486, 0.722, 1.0]
    darkOrange = [0.965, 0.584, 0.059, 1.0]

    # A decode array contains two elements for each component. In this
    # case, an image mask has one component so the array consists of
    # two values. When using this decode array, a sample value of 0
    # is mapped into the value 1, and the maximum sample value is
    # mapped into the value 0. This inverts the sense of the mask data.
    decode = (1, 0)

    # Create a Quartz data provider for the image data. Because this
    # data is static data, we don't need to release it so the data
    # release function is None.
    data = getMaskData1()
    dataProvider = Quartz.CGDataProviderCreateWithData(None, data, imageDataSize, None)
    if dataProvider is None:
        print("Couldn't create Mask1 Data provider!")
        return

    # Create a mask from the data.
    mask1 = Quartz.CGImageMaskCreate(
        width,
        height,
        bitsPerComponent,
        bitsPerPixel,
        bytesPerRow,
        dataProvider,
        None,
        shouldInterpolate,
    )
    # Create the same mask but with a decode array that
    # inverts the sense of the mask.
    invertedmask1 = Quartz.CGImageMaskCreate(
        width,
        height,
        bitsPerComponent,
        bitsPerPixel,
        bytesPerRow,
        dataProvider,
        decode,
        shouldInterpolate,
    )
    # Release the data provider now that this code no longer needs it.
    del dataProvider

    if mask1 is None or invertedmask1 is None:
        if mask1 is None:
            print("Couldn't create CGImageRef for the mask data 1!")

        if invertedmask1 is None:
            print("Couldn't create CGImageRef for the inverted mask data 1!")
        return

    # Get the pointer to the data for the second mask.
    data = getMaskData2()
    dataProvider = Quartz.CGDataProviderCreateWithData(None, data, imageDataSize, None)
    if dataProvider is None:
        print("Couldn't create Mask2 Data provider!")
        return

    mask2 = Quartz.CGImageMaskCreate(
        width,
        height,
        bitsPerComponent,
        bitsPerPixel,
        bytesPerRow,
        dataProvider,
        None,
        shouldInterpolate,
    )
    # Create the same mask but with a decode array that
    # inverts the sense of the mask.
    invertedmask2 = Quartz.CGImageMaskCreate(
        width,
        height,
        bitsPerComponent,
        bitsPerPixel,
        bytesPerRow,
        dataProvider,
        decode,
        shouldInterpolate,
    )
    # Release the data provider now that this code no longer needs it.
    del dataProvider
    if mask2 is None or invertedmask2 is None:
        if mask2 is None:
            print("Couldn't create CGImageRef for the mask data 2!")

        if invertedmask2 is None:
            print("Couldn't create CGImageRef for the inverted mask data 2!")

        return

    Quartz.CGContextScaleCTM(context, 1.5, 1.5)
    colorSpace = Utilities.getTheCalibratedRGBColorSpace()
    Quartz.CGContextSetFillColorSpace(context, colorSpace)

    # Set the fill color to a light blue.
    Quartz.CGContextSetFillColor(context, lightBlue)
    # Paint part of the background.
    backRect = Quartz.CGRectMake(width / 2, height / 2, width * 3, height)
    Quartz.CGContextFillRect(context, backRect)

    imageRect = Quartz.CGRectMake(0.0, height, width, height)
    Quartz.CGContextSaveGState(context)
    # Set the fill color to opaque black.
    Quartz.CGContextSetFillColor(context, black)
    # Mask 1.
    Quartz.CGContextDrawImage(context, imageRect, mask1)

    Quartz.CGContextTranslateCTM(context, width, 0)
    # Set the fill color to opaque red.
    Quartz.CGContextSetFillColor(context, darkRed)
    # Mask 2.
    Quartz.CGContextDrawImage(context, imageRect, mask2)
    Quartz.CGContextTranslateCTM(context, width, 0)
    # Set the fill color to dark orange.
    Quartz.CGContextSetFillColor(context, darkOrange)
    # Mask 3.
    Quartz.CGContextDrawImage(context, imageRect, mask1)

    Quartz.CGContextTranslateCTM(context, width, 0)
    # Make the orange 50% transparent.
    darkOrange[3] = 0.5
    Quartz.CGContextSetFillColor(context, darkOrange)
    # Mask 4.
    Quartz.CGContextDrawImage(context, imageRect, mask2)
    Quartz.CGContextRestoreGState(context)

    # Translate down the page. The cast is necessary
    # since height is typed as size_t which is unsigned.
    Quartz.CGContextTranslateCTM(context, 0, -height)

    # Set the fill color to an opaque green.
    Quartz.CGContextSetFillColor(context, darkGreen)
    # Mask 5.
    Quartz.CGContextDrawImage(context, imageRect, invertedmask2)

    Quartz.CGContextTranslateCTM(context, width, 0)
    # Set the fill color to a dark blue.
    Quartz.CGContextSetFillColor(context, darkBlue)
    # Mask 6.
    Quartz.CGContextDrawImage(context, imageRect, invertedmask1)
    Quartz.CGContextTranslateCTM(context, width, 0)
    # Set the fill color to purple.
    Quartz.CGContextSetFillColor(context, purple)
    # Mask 7.
    Quartz.CGContextDrawImage(context, imageRect, invertedmask2)
    Quartz.CGContextTranslateCTM(context, width, 0)

    # Make the purple 50% transparent.
    purple[3] = 0.5
    Quartz.CGContextSetFillColor(context, purple)
    # Mask 8.
    Quartz.CGContextDrawImage(context, imageRect, invertedmask1)


def doMaskImageWithMaskFromURL(
    context,
    imageURL,
    imagewidth,
    imageheight,
    bitsPerComponent,
    theMaskingImageURL,
    maskwidth,
    maskheight,
):

    imageBitsPerPixel = bitsPerComponent * 3
    bytesPerRow = ((imagewidth * imageBitsPerPixel) + 7) / 8
    shouldInterpolate = True
    imageDataProvider = Quartz.CGDataProviderCreateWithURL(imageURL)
    if imageDataProvider is None:
        print("Couldn't create Image Data provider!")
        return

    colorspace = Utilities.getTheCalibratedRGBColorSpace()
    image = Quartz.CGImageCreate(
        imagewidth,
        imageheight,
        bitsPerComponent,
        imageBitsPerPixel,
        bytesPerRow,
        colorspace,
        Quartz.kCGImageAlphaNone,
        imageDataProvider,
        None,
        shouldInterpolate,
        Quartz.kCGRenderingIntentDefault,
    )
    del imageDataProvider
    if image is None:
        print("Couldn't create CGImageRef for this data!")
        return

    imageRect = Quartz.CGRectMake(0.0, imageheight, imagewidth, imageheight)
    # Draw the image.
    Quartz.CGContextDrawImage(context, imageRect, image)

    # Now the mask.
    maskDataProvider = Quartz.CGDataProviderCreateWithURL(theMaskingImageURL)
    if maskDataProvider is None:
        print("Couldn't create Image Data provider!")
        return

    mask = Quartz.CGImageMaskCreate(
        maskwidth,
        maskheight,
        bitsPerComponent,
        bitsPerComponent,
        maskwidth,
        maskDataProvider,
        None,
        shouldInterpolate,
    )
    del maskDataProvider
    if mask is None:
        print("Couldn't create CGImageRef for mask data!")
        return

    # Draw the mask below the image. The current fill color (black)
    # is painted through the mask.
    maskRect = Quartz.CGRectMake(0.0, 0.0, maskwidth, maskheight)
    Quartz.CGContextDrawImage(context, maskRect, mask)

    # Create a new CGImage object, the image, masked with mask.
    imageMaskedWithImage = Quartz.CGImageCreateWithMask(image, mask)
    # Once the new image is created, the code can release the image
    # and the mask which make it up. Quartz retains what it needs
    # for the new masked image 'imageMaskedWithImage'.
    del image
    del mask
    if imageMaskedWithImage is None:
        print("Couldn't create image masked with mask!")
        return

    imageRect = Quartz.CGRectMake(
        imagewidth + 10, imageheight / 2, imagewidth, imageheight
    )
    # Draw the masked image to the right of the image and its mask.
    Quartz.CGContextDrawImage(context, imageRect, imageMaskedWithImage)


def doMaskImageWithGrayImageFromURL(
    context,
    imageURL,
    imagewidth,
    imageheight,
    bitsPerComponent,
    theMaskingImageURL,
    maskwidth,
    maskheight,
):

    imageBitsPerPixel = bitsPerComponent * 3
    bytesPerRow = ((imagewidth * imageBitsPerPixel) + 7) / 8
    shouldInterpolate = True

    imageDataProvider = Quartz.CGDataProviderCreateWithURL(imageURL)
    if imageDataProvider is None:
        print("Couldn't create Image Data provider!")
        return

    colorspace = Utilities.getTheCalibratedRGBColorSpace()
    image = Quartz.CGImageCreate(
        imagewidth,
        imageheight,
        bitsPerComponent,
        imageBitsPerPixel,
        bytesPerRow,
        colorspace,
        Quartz.kCGImageAlphaNone,
        imageDataProvider,
        None,
        shouldInterpolate,
        Quartz.kCGRenderingIntentDefault,
    )
    del imageDataProvider
    if image is None:
        print("Couldn't create CGImageRef for this data!")
        return

    imageRect = Quartz.CGRectMake(0.0, imageheight, imagewidth, imageheight)
    # Draw the image.
    Quartz.CGContextDrawImage(context, imageRect, image)

    # Now the mask.
    maskDataProvider = Quartz.CGDataProviderCreateWithURL(theMaskingImageURL)
    if maskDataProvider is None:
        print("Couldn't create Image Data provider!")
        return

    # The color space for the image MUST be DeviceGray for it to
    # be used as a masking image with CGImageCreateWithMask.
    deviceGraySpace = Quartz.CGColorSpaceCreateDeviceGray()
    mask = Quartz.CGImageCreate(
        maskwidth,
        maskheight,
        bitsPerComponent,
        bitsPerComponent,
        maskwidth,
        deviceGraySpace,
        Quartz.kCGImageAlphaNone,
        maskDataProvider,
        None,
        shouldInterpolate,
        Quartz.kCGRenderingIntentDefault,
    )
    # Release the color space since it is no longer needed.
    del deviceGraySpace
    del maskDataProvider

    if mask is None:
        print("Couldn't create CGImageRef for gray image data!")
        return

    # Draw the mask below the image. The current fill color (black)
    # is painted through the mask.
    maskRect = Quartz.CGRectMake(0.0, 0.0, maskwidth, maskheight)
    Quartz.CGContextDrawImage(context, maskRect, mask)

    # Create a new CGImage object, the image, masked with mask.
    imageMaskedWithImage = Quartz.CGImageCreateWithMask(image, mask)

    # Once the new image is created, the code can release the image
    # and the mask which make it up. Quartz retains what it needs
    # for the new masked image 'imageMaskedWithImage'.
    del image
    del mask

    if imageMaskedWithImage is None:
        print("Couldn't create image masked with mask!")
        return

    imageRect = Quartz.CGRectMake(
        imagewidth + 10, imageheight / 2, imagewidth, imageheight
    )
    # Draw the masked image to the right of the image and its mask.
    Quartz.CGContextDrawImage(context, imageRect, imageMaskedWithImage)
    # Be sure and release the masked image.
    del imageMaskedWithImage


def doMaskImageWithColorFromURL(context, url, width, height, isColor):

    # This routine treats color images as RGB.
    bitsPerComponent = 8
    if isColor:
        bitsPerPixel = bitsPerComponent * 3
    else:
        bitsPerPixel = bitsPerComponent

    bytesPerRow = ((width * bitsPerPixel) + 7) / 8
    shouldInterpolate = True

    # This is a range of dark gray to black colors for an 8 bit per component
    # image in a gray or RGB color space. The entries are image sample
    # values of 0-0x1F for the first color component, 0-0x1F for the
    # second color component, and so on. For image sample values where
    # all components fall within the ranges in maskingColors, the sample
    # value is masked and therefore unpainted.
    maskingColors = (0x00, 0x1F, 0x00, 0x1F, 0x00, 0x1F)
    backColor = (1.0, 0.0, 0.0, 1.0)  # Opaque red.

    # Create a Quartz data provider from the supplied URL.
    dataProvider = Quartz.CGDataProviderCreateWithURL(url)
    if dataProvider is None:
        print("Couldn't create Image data provider!")
        return

    # Create an image of the specified width, height and bits per pixel
    # from the URL.
    if isColor:
        colorspace = Utilities.getTheCalibratedRGBColorSpace()
    else:
        colorspace = Utilities.getTheCalibratedGrayColorSpace()

    image = Quartz.CGImageCreate(
        width,
        height,
        bitsPerComponent,
        bitsPerPixel,
        bytesPerRow,
        colorspace,
        Quartz.kCGImageAlphaNone,
        dataProvider,
        None,
        shouldInterpolate,
        Quartz.kCGRenderingIntentDefault,
    )
    del dataProvider
    if image is None:
        print("Couldn't create CGImageRef for this data!")
        return

    imageRect = Quartz.CGRectMake(10.0, 10.0, width, height)
    # Quartz.CGContextScaleCTM(context, 0.33, 0.33)
    # Set the color space and the color, then
    # paint a red rectangle behind the image.
    Quartz.CGContextSetFillColorSpace(context, colorspace)
    Quartz.CGContextSetFillColor(context, backColor)
    Quartz.CGContextFillRect(context, imageRect)
    # Draw the image into the rectangle.
    Quartz.CGContextDrawImage(context, imageRect, image)
    # Create a new image from the original one, masking out a range
    # of the blackest sample values.
    imageMaskedWithColor = Quartz.CGImageCreateWithMaskingColors(image, maskingColors)
    # Release the original image; it is no longer needed.
    del image
    if imageMaskedWithColor is None:
        print("Couldn't create CGImageRef for masking color!")
        return

    # Paint the rectangle behind the next image with red.
    imageRect = Quartz.CGRectMake(30.0 + width, 10.0, width, height)
    Quartz.CGContextFillRect(context, imageRect)
    # Draw the image. Image sample values in the range of
    # the masking color are unpainted, allowing the background
    # to show through.
    Quartz.CGContextDrawImage(context, imageRect, imageMaskedWithColor)


if 1:  # Set to 1 for code in the book.

    def drawWithClippingMask(context, theMaskingImageURL, imagewidth, imageheight):
        # An array of CGColor objects.
        colors = (
            Utilities.getRGBOpaqueDarkGreenColor(),
            Utilities.getRGBOpaqueDarkBlueColor(),
            Utilities.getRGBOpaqueBlueColor(),
            Utilities.getRGBOpaqueRedColor(),
        )

        imageBitsPerComponent = 8
        bytesPerRow = imagewidth
        shouldInterpolate = True
        decode = (1, 0)

        # Create the data.
        dataProvider = Quartz.CGDataProviderCreateWithURL(theMaskingImageURL)
        if dataProvider is None:
            print("Couldn't create Image data provider!")
            return

        cs = Quartz.CGColorSpaceCreateDeviceGray()
        image = Quartz.CGImageCreate(
            imagewidth,
            imageheight,
            imageBitsPerComponent,
            imageBitsPerComponent,
            bytesPerRow,
            cs,
            Quartz.kCGImageAlphaNone,
            dataProvider,
            decode,
            shouldInterpolate,
            Quartz.kCGRenderingIntentDefault,
        )
        del cs
        del dataProvider

        if image is None:
            print("Couldn't create Image!")
            return

        imageRect = Quartz.CGRectMake(0, 0, imagewidth * 2 / 3, imageheight * 2 / 3)

        # Position for drawing the image at the left side of the figure.
        Quartz.CGContextTranslateCTM(context, 50, 50)

        # Draw the image.
        Quartz.CGContextDrawImage(context, imageRect, image)

        # Position to the right of the image just painted.
        Quartz.CGContextTranslateCTM(context, Quartz.CGRectGetWidth(imageRect) + 25, 0)

        # Clip to the image.
        Quartz.CGContextClipToMask(context, imageRect, image)
        # Release the image since this code no longer needs it.
        del image

        # Make a rect that has a width and height 1/3 that of the image.
        rect = Quartz.CGRectMake(
            0,
            0,
            Quartz.CGRectGetWidth(imageRect) / 3,
            Quartz.CGRectGetHeight(imageRect) / 3,
        )

        Quartz.CGContextTranslateCTM(context, 0, 2 * Quartz.CGRectGetHeight(rect))

        # Draw a 3 x 3 grid of rectangles, setting the color for each rectangle
        # by cycling through the array of CGColor objects in the 'colors' array.
        for j in range(3):
            Quartz.CGContextSaveGState(context)
            for i in range(3):
                # Draw a row of rectangles.
                # Set the fill color using one of the CGColor objects in the
                # colors array.
                Quartz.CGContextSetFillColorWithColor(context, colors[(i + j) % 4])
                Quartz.CGContextFillRect(context, rect)
                Quartz.CGContextTranslateCTM(context, Quartz.CGRectGetWidth(rect), 0)

            Quartz.CGContextRestoreGState(context)
            # Position to draw the next row.
            Quartz.CGContextTranslateCTM(context, 0, -Quartz.CGRectGetHeight(rect))


else:
    # This code works just fine to screen but when drawing to a PDF
    # or printing context the masked drawing is completely masked out
    # due to a bug in Quartz prior to Tiger 10.4.3.
    def drawWithClippingMask(context, theMaskingImageURL, maskwidth, maskheight):
        # An array of Quartz.CGColor objects.
        colors = (
            Utilities.getRGBOpaqueDarkGreenColor(),
            Utilities.getRGBOpaqueDarkBlueColor(),
            Utilities.getRGBOpaqueBlueColor(),
            Utilities.getRGBOpaqueRedColor(),
        )
        maskBitsPerComponent = 8
        # bytesPerRow = ((maskwidth * maskBitsPerComponent) + 7) / 8
        shouldInterpolate = True
        maskDataProvider = Quartz.CGDataProviderCreateWithURL(theMaskingImageURL)

        if maskDataProvider is None:
            print("Couldn't create Image Mask provider!")
            return
        mask = Quartz.CGImageMaskCreate(
            maskwidth,
            maskheight,
            maskBitsPerComponent,
            maskBitsPerComponent,
            maskwidth,
            maskDataProvider,
            None,
            shouldInterpolate,
        )
        del maskDataProvider

        if mask is None:
            print("Couldn't create Image Mask!")
            return

        maskRect = Quartz.CGRectMake(0, 0, maskwidth / 3, maskheight / 3)

        # Position for drawing the mask at the left side of the figure.
        Quartz.CGContextTranslateCTM(context, 50, 50)
        # Set the context fill color to a Quartz.CGColor object that is black.
        Quartz.CGContextSetFillColorWithColor(
            context, Utilities.getRGBOpaqueBlackColor()
        )
        # Draw the mask. It is painted with with the black fill color.
        Quartz.CGContextDrawImage(context, maskRect, mask)

        # Position to the right of the mask just painted.
        Quartz.CGContextTranslateCTM(context, Quartz.CGRectGetWidth(maskRect) + 25, 0)

        # Clip to the mask.
        Quartz.CGContextClipToMask(context, maskRect, mask)
        # Release the mask since this code no longer needs it.
        del mask

        # Make a rect that has a width and height 1/3 that of the image mask.
        rect = Quartz.CGRectMake(
            0,
            0,
            Quartz.CGRectGetWidth(maskRect) / 3,
            Quartz.CGRectGetHeight(maskRect) / 3,
        )

        Quartz.CGContextTranslateCTM(context, 0, 2 * Quartz.CGRectGetHeight(rect))

        # Draw a 3 x 3 grid of rectangles, setting the color for each rectangle
        # by cycling through the array of CGColor objects in the 'colors' array.
        for j in range(3):
            Quartz.CGContextSaveGState(context)
            for i in range(3):
                # Draw a row of rectangles.
                # Set the fill color using one of the CGColor objects in the
                # colors array.
                Quartz.CGContextSetFillColorWithColor(context, colors[(i + j) % 4])
                Quartz.CGContextFillRect(context, rect)
                Quartz.CGContextTranslateCTM(context, Quartz.CGRectGetWidth(rect), 0)
            Quartz.CGContextRestoreGState(context)
            # Position to draw the next row.
            Quartz.CGContextTranslateCTM(context, 0, -Quartz.CGRectGetHeight(rect))



import Quartz
import Utilities


def scalePatternPhase(phase):
    # Adjust the pattern phase if scaling to export as bits. This is equivalent to scaling base
    # space by the scaling factor.
    patternScaling = Utilities.getScalingFactor()
    if patternScaling != 1.0:
        phase = Quartz.CGSizeApplyAffineTransform(
            phase, Quartz.CGAffineTransformMakeScale(patternScaling, patternScaling)
        )

    return phase


def scalePatternMatrix(patternTransform):
    # Scale the pattern by the scaling factor when exporting to bits. This is equivalent to
    # scaling base space by the scaling factor.
    patternScaling = Utilities.getScalingFactor()
    if patternScaling != 1.0:
        patternTransform = Quartz.CGAffineTransformConcat(
            patternTransform,
            Quartz.CGAffineTransformMakeScale(patternScaling, patternScaling),
        )

    return patternTransform


def myDrawRedBlackCheckerBoardPattern(info, patternCellContext):
    # This pattern proc draws a red and a black rectangle
    # patch representing the minimum cell needed to paint a
    # checkerboard with that pattern.
    #
    # Each 'cell' of the checkerboard is 2 units on a side.
    #
    # This code uses Quartz.CGColorRefs which are available in Panther
    # and later only. Patterns are available in all versions of
    # macOS but this code uses Quartz.CGColorRefs for convenience
    # and efficiency.

    # Paint a black checkerboard box.
    Quartz.CGContextSetFillColorWithColor(
        patternCellContext, Utilities.getRGBOpaqueBlackColor()
    )
    # This is a 1x1 unit rect whose origin is at 0,0 in pattern space.
    Quartz.CGContextFillRect(patternCellContext, Quartz.CGRectMake(0.0, 0.0, 1.0, 1.0))
    # This is a 1x1 unit rect whose origin is at 1,1 in pattern space.
    Quartz.CGContextFillRect(patternCellContext, Quartz.CGRectMake(1.0, 1.0, 1.0, 1.0))

    # Paint a red checkerboard box.
    Quartz.CGContextSetFillColorWithColor(
        patternCellContext, Utilities.getRGBOpaqueRedColor()
    )
    # This is a 1x1 unit rect whose origin is at 1,0 in pattern space,
    # that is, immediately to the right of first black checkerboard box.
    Quartz.CGContextFillRect(patternCellContext, Quartz.CGRectMake(1.0, 0.0, 1.0, 1.0))
    # This is a 1x1 unit rect whose origin is at 0,1 in pattern space,
    # that is, immediately above the first black checkerboard box.
    Quartz.CGContextFillRect(patternCellContext, Quartz.CGRectMake(0.0, 1.0, 1.0, 1.0))


def createRedBlackCheckerBoardPattern(patternTransform):
    pattern = Quartz.CGPatternCreate(
        None,
        # The pattern cell origin is at (0,0) with a
        # width of 2 units and a height of 2 units.
        Quartz.CGRectMake(0, 0, 2, 2),
        # Use the pattern transform supplied to this routine.
        scalePatternMatrix(patternTransform),
        # In pattern space the xStep is 2 units to the next cell in x
        # and the yStep is 2 units to the next row of cells in y.
        2,
        2,
        # This value is a good choice for this type of pattern and it
        # avoids seams between tiles.
        Quartz.kCGPatternTilingConstantSpacingMinimalDistortion,
        # This pattern has intrinsic color.
        True,
        myDrawRedBlackCheckerBoardPattern,
    )
    return pattern


def doRedBlackCheckerboard(context):
    dash = [4]
    pattern = createRedBlackCheckerBoardPattern(
        Quartz.CGAffineTransformMakeScale(20, 20)
    )
    if pattern is None:
        print("Couldn't create pattern!")
        return

    # Create the pattern color space. Since the pattern
    # itself has intrinsic color, the 'baseColorSpace' parameter
    # to Quartz.CGColorSpaceCreatePattern must be None.
    patternColorSpace = Quartz.CGColorSpaceCreatePattern(None)
    Quartz.CGContextSetFillColorSpace(context, patternColorSpace)

    # The pattern has intrinsic color so the color components array
    # passed to CGContextSetFillPattern is just the alpha value used
    # to composite the pattern cell.

    # Paint the pattern with alpha = 1.
    color = [1.0]

    # Set the fill color to the checkerboard pattern.
    Quartz.CGContextSetFillPattern(context, pattern, color)

    # Fill a 100x100 unit rect at (20,20).
    Quartz.CGContextFillRect(context, Quartz.CGRectMake(20, 20, 100, 100))

    # Save the graphics state before changing the stroke color.
    Quartz.CGContextSaveGState(context)
    if 1:
        # Set the stroke color space and color to the pattern.
        Quartz.CGContextSetStrokeColorSpace(context, patternColorSpace)
        Quartz.CGContextSetStrokePattern(context, pattern, color)

        # Stroke an ellipse with the pattern.
        Quartz.CGContextSetLineWidth(context, 8)
        Quartz.CGContextBeginPath(context)
        Utilities.myCGContextAddEllipseInRect(
            context, Quartz.CGRectMake(120, 20, 50, 100)
        )
        Quartz.CGContextStrokePath(context)

    # Restore to the graphics state without the
    # pattern stroke color.
    Quartz.CGContextRestoreGState(context)

    # Now draw text.
    Quartz.CGContextSetTextMatrix(context, Quartz.CGAffineTransformIdentity)
    # Choose the font with the PostScript name "Times-Roman",
    # size 80 points, with the encoding MacRoman encoding.
    Quartz.CGContextSelectFont(context, b"Times-Roman", 80, Quartz.kCGEncodingMacRoman)

    # Using the fill text drawing mode.
    Quartz.CGContextSetTextDrawingMode(context, Quartz.kCGTextFill)

    # Draw text with the pattern.
    Quartz.CGContextShowTextAtPoint(context, 20, 120, b"Text", 4)

    # Rectangle 1, filled.
    Quartz.CGContextFillRect(context, Quartz.CGRectMake(200, 20, 90, 90))

    # Rectangle 2, filled and stroked with a dash.
    Quartz.CGContextSetLineWidth(context, 2)
    Quartz.CGContextSetLineDash(context, 0, dash, 1)
    Quartz.CGContextBeginPath(context)
    Quartz.CGContextAddRect(context, Quartz.CGRectMake(200, 70, 90, 90))
    Quartz.CGContextDrawPath(context, Quartz.kCGPathFillStroke)


def doPatternMatrix(context):
    basePatternMatrix = Quartz.CGAffineTransformMakeScale(20, 20)
    pattern = createRedBlackCheckerBoardPattern(basePatternMatrix)
    if pattern is None:
        print("Couldn't create pattern!")
        return

    # Create the pattern color space. Since the pattern
    # itself has intrinsic color, the 'baseColorSpace' parameter
    # to Quartz.CGColorSpaceCreatePattern must be None.
    patternColorSpace = Quartz.CGColorSpaceCreatePattern(None)

    Quartz.CGContextSetFillColorSpace(context, patternColorSpace)
    del patternColorSpace

    Quartz.CGContextTranslateCTM(context, 40, 40)
    Quartz.CGContextSetPatternPhase(context, scalePatternPhase(Quartz.CGSize(40, 40)))

    # The pattern has intrinsic color so the color components array
    # passed to Quartz.CGContextSetFillPattern is the alpha value used
    # to composite the pattern cell.

    # Paint the pattern first with alpha = 1.
    color = [1]
    Quartz.CGContextSetFillPattern(context, pattern, color)

    # Rectangle 1.
    Quartz.CGContextFillRect(context, Quartz.CGRectMake(0, 0, 100, 100))

    Quartz.CGContextSaveGState(context)
    if 1:
        # Rectangle 2.
        # Paint the pattern with 65% alpha.
        color = [0.65]
        Quartz.CGContextSetFillPattern(context, pattern, color)
        # Rotate 45 degrees about the point (150, 50).
        Quartz.CGContextTranslateCTM(context, 150.0, 50.0)
        Quartz.CGContextRotateCTM(context, Utilities.DEGREES_TO_RADIANS(45.0))
        Quartz.CGContextTranslateCTM(context, -50.0, -50.0)
        # Rectangle 2. Patterns do not translate, scale or
        # rotate with the CTM. You can see that the pattern
        # tile of this filled rectangle is that of Rectangle
        # 1.
        Quartz.CGContextFillRect(context, Quartz.CGRectMake(0, 0, 100, 100))
        # Release the pattern.
        del pattern
    Quartz.CGContextRestoreGState(context)

    Quartz.CGContextSaveGState(context)
    if 1:
        # Rectangle 3. The pattern is rotated with the object.
        # Rotate 45 degrees about the point 250, 50.
        t = Quartz.CGAffineTransformMakeTranslation(250.0, 50.0)
        t = Quartz.CGAffineTransformRotate(t, Utilities.DEGREES_TO_RADIANS(45.0))
        # Translate back to -50, -50.
        t = Quartz.CGAffineTransformTranslate(t, -50.0, -50.0)
        Quartz.CGContextConcatCTM(context, t)
        # Make a new pattern that is equivalent to
        # the old pattern but transformed to current user
        # space. The order of transformations is crucial.
        # This ordering is equivalent to using the same pattern
        # matrix as before but transforming base space by t.
        patTransform = Quartz.CGAffineTransformConcat(basePatternMatrix, t)
        pattern = createRedBlackCheckerBoardPattern(patTransform)
        color = [1]
        Quartz.CGContextSetFillPattern(context, pattern, color)
        # Release the pattern.
        del pattern
        Quartz.CGContextFillRect(context, Quartz.CGRectMake(0, 0, 100, 100))
    Quartz.CGContextRestoreGState(context)

    Quartz.CGContextSaveGState(context)
    if 1:
        # Rectangle 4. The pattern is scaled with the object.
        # Translate and scale.
        t = Quartz.CGAffineTransformMakeTranslation(320, 0)
        t = Quartz.CGAffineTransformScale(t, 2, 2)
        Quartz.CGContextConcatCTM(context, t)
        # Make a new pattern that is equivalent to
        # the old pattern but transformed to current user
        # space. The order of transformations is crucial.
        # This ordering is equivalent to using the same pattern
        # matrix as before but transforming base space by t.
        patTransform = Quartz.CGAffineTransformConcat(basePatternMatrix, t)
        pattern = createRedBlackCheckerBoardPattern(patTransform)
        color = [1]
        Quartz.CGContextSetFillPattern(context, pattern, color)
        # Release the pattern.
        del pattern
        Quartz.CGContextFillRect(context, Quartz.CGRectMake(0, 0, 100, 100))
    Quartz.CGContextRestoreGState(context)


def doPatternPhase(context):
    pattern = createRedBlackCheckerBoardPattern(
        Quartz.CGAffineTransformMakeScale(20, 20)
    )
    if pattern is None:
        print("Couldn't create pattern!")
        return

    # Create the pattern color space for a colored pattern.
    patternColorSpace = Quartz.CGColorSpaceCreatePattern(None)
    Quartz.CGContextSetFillColorSpace(context, patternColorSpace)

    # Paint the pattern with alpha = 1.
    color = (1,)
    Quartz.CGContextSetFillPattern(context, pattern, color)

    # Rectangle 1
    Quartz.CGContextFillRect(context, Quartz.CGRectMake(20, 150, 100, 100))

    # Rectangle 2
    Quartz.CGContextFillRect(context, Quartz.CGRectMake(130, 150, 100, 100))

    # Rectangle 3
    # Set the pattern phase so that the pattern origin
    # is at the lower-left of the shape.
    Quartz.CGContextSetPatternPhase(
        context, scalePatternPhase(Quartz.CGSizeMake(20, 20))
    )
    Quartz.CGContextFillRect(context, Quartz.CGRectMake(20, 20, 100, 100))

    # Rectangle 4
    # Set the pattern phase so that the pattern origin
    # is at the lower-left corner of the shape.
    Quartz.CGContextSetPatternPhase(
        context, scalePatternPhase(Quartz.CGSizeMake(130, 20))
    )
    Quartz.CGContextTranslateCTM(context, 130, 20)
    Quartz.CGContextFillRect(context, Quartz.CGRectMake(0, 0, 100, 100))


def drawRotatedRect(c, p):
    r = Quartz.CGRectMake(0, 0, 1, 1)
    Quartz.CGContextSaveGState(c)
    if 1:
        Quartz.CGContextTranslateCTM(c, p.x, p.y)
        Quartz.CGContextRotateCTM(c, Utilities.DEGREES_TO_RADIANS(45))
        Quartz.CGContextTranslateCTM(c, -r.size.width / 2, -r.size.height / 2)
        Quartz.CGContextFillRect(c, r)
    Quartz.CGContextRestoreGState(c)


def myStencilPatternProc(info, patternCellContext):
    drawRotatedRect(patternCellContext, Quartz.CGPointMake(1, 1))
    drawRotatedRect(patternCellContext, Quartz.CGPointMake(1.75, 1))


def createStencilPattern(patternTransform):
    pattern = Quartz.CGPatternCreate(
        None,
        # The pattern cell origin is at (0,0) with a
        # width of 2.5 units and a height of 2 units. This
        # pattern cell has transparent areas since
        # the pattern proc only marks a portion of the cell.
        Quartz.CGRectMake(0, 0, 2.5, 2),
        # Use the pattern transform supplied to this routine.
        scalePatternMatrix(patternTransform),
        # Use the width and height of the pattern cell for
        # the xStep and yStep.
        2.5,
        2,
        # This value is a good choice for this type of pattern and it
        # avoids seams between tiles.
        Quartz.kCGPatternTilingConstantSpacingMinimalDistortion,
        # This pattern does not have intrinsic color.
        False,  # Must be False for a stencil pattern.
        myStencilPatternProc,
    )
    return pattern


def doStencilPattern(context):
    pattern = createStencilPattern(Quartz.CGAffineTransformMakeScale(20, 20))
    if pattern is None:
        print("Couldn't create pattern!")
        return

    # Create the pattern color space. This pattern is a stencil
    # pattern so when the code sets the pattern it also sets the
    # color it will paint the pattern with. In order to
    # set the pattern color space in this case we also have
    # to say what underlying color space should be used when
    # the pattern proc is called.
    baseColorSpace = Utilities.getTheCalibratedRGBColorSpace()
    patternColorSpace = Quartz.CGColorSpaceCreatePattern(baseColorSpace)

    Quartz.CGContextSetFillColorSpace(context, patternColorSpace)
    # This code is finished with the pattern color space and can release
    # it because Quartz retains it while it is the current color space.
    del patternColorSpace

    # The pattern has no intrinsic color so the color components array
    # passed to CGContextSetFillPattern contains the colors to paint
    # the pattern with in the baseColorSpace. In the case here,
    # first paint the pattern with opaque blue.
    color = (0.11, 0.208, 0.451, 1.0)
    Quartz.CGContextSetFillPattern(context, pattern, color)

    # Rectangle 1.
    Quartz.CGContextSetPatternPhase(
        context, scalePatternPhase(Quartz.CGSizeMake(20, 160))
    )
    Quartz.CGContextBeginPath(context)
    Quartz.CGContextAddRect(context, Quartz.CGRectMake(20, 160, 105, 80))
    Quartz.CGContextDrawPath(context, Quartz.kCGPathFillStroke)

    # Rectangle 2.
    # Set the pattern color so the stencil pattern
    # is painted in a yellow shade.
    color = (1.0, 0.816, 0.0, 1.0)
    Quartz.CGContextSetFillPattern(context, pattern, color)
    # Set the pattern phase to the origin of the next object.
    Quartz.CGContextSetPatternPhase(
        context, scalePatternPhase(Quartz.CGSizeMake(140, 160))
    )
    Quartz.CGContextBeginPath(context)
    Quartz.CGContextAddRect(context, Quartz.CGRectMake(140, 160, 105, 80))
    Quartz.CGContextDrawPath(context, Quartz.kCGPathFillStroke)

    Quartz.CGContextSaveGState(context)
    if 1:
        Quartz.CGContextSetFillColorWithColor(
            context, Utilities.getRGBOpaqueBlueColor()
        )
        # Fill color is now blue. Paint two blue rectangles
        # that will be underneath the drawing which follows.
        Quartz.CGContextFillRect(context, Quartz.CGRectMake(20, 40, 105, 80))
        Quartz.CGContextFillRect(context, Quartz.CGRectMake(140, 40, 105, 80))
    Quartz.CGContextRestoreGState(context)

    # The fill color is again the stencil pattern with
    # the underlying fill color an opaque yellow.

    # Rectangle 3.
    # This paints over the blue rect just painted at 20,40
    # and the blue underneath is visible where the pattern has
    # transparent areas.
    Quartz.CGContextSetPatternPhase(
        context, scalePatternPhase(Quartz.CGSizeMake(20, 40))
    )
    Quartz.CGContextFillRect(context, Quartz.CGRectMake(20, 40, 105, 80))

    # Rectangle 4.
    # Change the alpha value of the underlying color used
    # to paint the stencil pattern.
    color = list(color)
    color[3] = 0.75
    Quartz.CGContextSetFillPattern(context, pattern, color)
    Quartz.CGContextSetPatternPhase(
        context, scalePatternPhase(Quartz.CGSizeMake(140, 40))
    )
    Quartz.CGContextFillRect(context, Quartz.CGRectMake(140, 40, 105, 80))


class MyPDFPatternInfo(object):
    rect = None
    pdfDoc = None


def myDrawPDFPattern(info, patternCellContext):
    # This pattern proc draws the first page of a PDF document to
    # a destination rect.
    Quartz.CGContextSaveGState(patternCellContext)
    Quartz.CGContextClipToRect(patternCellContext, info.rect)
    Quartz.CGContextDrawPDFDocument(patternCellContext, info.rect, info.pdfDoc, 1)
    Quartz.CGContextRestoreGState(patternCellContext)


# Versions of Tiger prior to 10.4.3 have a bug such that use of an xStep that
# doesn't match the width of pattern bounding box or a yStep that doesn't match the
# height of the pattern bounding box produces incorrect results when drawn
# to a bit-based context. Setting TIGERSTEPWORKAROUND works around this bug.


TIGERSTEPWORKAROUND = 1
SCALEPATTERN = 1
OPTIMIZEDPERF = 0


def createPDFPatternPattern(additionalTransformP, url):
    patternInfoP = MyPDFPatternInfo()

    patternInfoP.pdfDoc = Quartz.CGPDFDocumentCreateWithURL(url)
    if patternInfoP.pdfDoc is None:
        print("Couldn't create PDF document reference!")
        return

    patternInfoP.rect = Quartz.CGPDFDocumentGetMediaBox(patternInfoP.pdfDoc, 1)
    # Set the origin of the media rect for the PDF document to (0,0).
    patternInfoP.rect.origin = Quartz.CGPointZero

    if additionalTransformP is not None:
        patternTransform = additionalTransformP
    else:
        patternTransform = Quartz.CGAffineTransformIdentity

    # To emulate the example from the bitmap context drawing chapter,
    # the tile offset in each dimension is the tile size in that
    # dimension, plus 6 units.
    if SCALEPATTERN:
        tileOffsetX = 6.0 + patternInfoP.rect.size.width
        tileOffsetY = 6.0 + patternInfoP.rect.size.height
    else:
        tileOffsetX = 2.0 + patternInfoP.rect.size.width
        tileOffsetY = 2.0 + patternInfoP.rect.size.height

    # Tiger versions 10.4.0 - 10.4.2 have a bug such that the bounds
    # width and height is incorrectly used as the xstep,ystep.
    # To workaround this bug, we can make the bounds rect incorporate
    # the xstep,ystep since xstep,ystep are larger than the bounds.
    if OPTIMIZEDPERF or TIGERSTEPWORKAROUND:
        patternRect = Quartz.CGRectMake(0, 0, tileOffsetX, tileOffsetY)
    else:
        patternRect = patternInfoP.rect

    if OPTIMIZEDPERF:
        # Produces best performance if bbox == xstep/ystep
        spacing = Quartz.kCGPatternTilingConstantSpacing
    else:
        spacing = Quartz.kCGPatternTilingConstantSpacingMinimalDistortion

    pattern = Quartz.CGPatternCreate(
        patternInfoP,
        # The pattern cell size is the size
        # of the media rect of the PDF document.
        patternRect,
        scalePatternMatrix(patternTransform),
        tileOffsetX,
        tileOffsetY,
        # This value is a good choice for this type of pattern and
        #  it avoids seams between tiles.
        spacing,
        # This pattern has intrinsic color.
        True,
        myDrawPDFPattern,
    )
    # If the pattern can't be created then release the
    # pattern resources and info parameter.
    if pattern is None:
        patternInfoP = None

    return pattern


def drawWithPDFPattern(context, url):
    if SCALEPATTERN:
        patternMatrix = Quartz.CGAffineTransformMakeScale(1.0 / 3, 1.0 / 3)
    else:
        patternMatrix = Quartz.CGAffineTransformMakeScale(1, 1)

    # Scale the PDF pattern down to 1/3 its original size.
    pdfPattern = createPDFPatternPattern(patternMatrix, url)
    if pdfPattern is None:
        print("Couldn't create pattern!")
        return

    # Create the pattern color space. Since the pattern
    # itself has intrinsic color, the 'baseColorSpace' parameter
    # to CGColorSpaceCreatePattern must be None.
    patternColorSpace = Quartz.CGColorSpaceCreatePattern(None)
    Quartz.CGContextSetFillColorSpace(context, patternColorSpace)
    # Quartz retains the color space so this code
    # can now release it since it no longer needs it.
    del patternColorSpace

    # Paint the pattern with an alpha of 1.
    color = (1,)
    Quartz.CGContextSetFillPattern(context, pdfPattern, color)
    # Quartz retains the pattern so this code
    # can now release it since it no longer needs it.
    del pdfPattern

    # Fill a US Letter size rect with the pattern.
    Quartz.CGContextFillRect(context, Quartz.CGRectMake(0, 0, 612, 792))



"""
SampleCIView - simple OpenGL based CoreImage view
"""

# XXX: FIXME
# flake8: noqa F403, F405

import CGL
import Cocoa
import objc
import Quartz
from OpenGL.GL import *
from OpenGL.GL.APPLE.transform_hint import *

# The default pixel format
_pf = None


class SampleCIView(Cocoa.NSOpenGLView):
    _context = objc.ivar()
    _image = objc.ivar()
    _lastBounds = objc.ivar(type=Cocoa.NSRect.__typestr__)

    @classmethod
    def defaultPixelFormat(self):
        global _pf

        if _pf is None:
            # Making sure the context's pixel format doesn't have a recovery
            # renderer is important - otherwise CoreImage may not be able to
            # create deeper context's that share textures with this one.

            attr = (
                Cocoa.NSOpenGLPFAAccelerated,
                Cocoa.NSOpenGLPFANoRecovery,
                Cocoa.NSOpenGLPFAColorSize,
                32,
            )
            _pf = Cocoa.NSOpenGLPixelFormat.alloc().initWithAttributes_(attr)

        return _pf

    def image(self):
        return self._image

    def setImage_dirtyRect_(self, image, r):
        if self._image is not image:
            self._image = image

            if Quartz.CGRectIsInfinite(r):
                self.setNeedsDisplay_(True)
            else:
                self.setNeedsDisplayInRect_(r)

    def setImage_(self, image):
        self.setImage_dirtyRect_(image, Quartz.CGRectInfinite)

    def prepareOpenGL(self):
        parm = 1

        # Enable beam-synced updates.

        self.openGLContext().setValues_forParameter_(
            (parm,), Cocoa.NSOpenGLCPSwapInterval
        )

        # Make sure that everything we don't need is disabled. Some of these
        # are enabled by default and can slow down rendering.

        glDisable(GL_ALPHA_TEST)
        glDisable(GL_DEPTH_TEST)
        glDisable(GL_SCISSOR_TEST)
        glDisable(GL_BLEND)
        glDisable(GL_DITHER)
        glDisable(GL_CULL_FACE)
        glColorMask(GL_TRUE, GL_TRUE, GL_TRUE, GL_TRUE)
        glDepthMask(GL_FALSE)
        glStencilMask(0)
        glClearColor(0.0, 0.0, 0.0, 0.0)
        glHint(GL_TRANSFORM_HINT_APPLE, GL_FASTEST)

    def viewBoundsDidChange_(self, bounds):
        # For subclasses.
        pass

    def updateMatrices(self):
        r = self.bounds()

        if r != self._lastBounds:
            self.openGLContext().update()

            # Install an orthographic projection matrix (no perspective)
            # with the origin in the bottom left and one unit equal to one
            # device pixel.

            glViewport(0, 0, r.size.width, r.size.height)

            glMatrixMode(GL_PROJECTION)
            glLoadIdentity()
            glOrtho(0, r.size.width, 0, r.size.height, -1, 1)

            glMatrixMode(GL_MODELVIEW)
            glLoadIdentity()

            self._lastBounds = r

            self.viewBoundsDidChange_(r)

    def drawRect_(self, r):
        self.openGLContext().makeCurrentContext()

        # Allocate a CoreImage rendering context using the view's OpenGL
        # context as its destination if none already exists.

        if self._context is None:
            pf = self.pixelFormat()
            if pf is None:
                pf = type(self).defaultPixelFormat()

            self._context = Quartz.CIContext.contextWithCGLContext_pixelFormat_options_(
                CGL.CGLGetCurrentContext(), pf.CGLPixelFormatObj(), None
            )

        ir = Quartz.CGRectIntegral(r)

        if Cocoa.NSGraphicsContext.currentContextDrawingToScreen():
            self.updateMatrices()

            # Clear the specified subrect of the OpenGL surface then
            # render the image into the view. Use the GL scissor test to
            # clip to * the subrect. Ask CoreImage to generate an extra
            # pixel in case * it has to interpolate (allow for hardware
            # inaccuracies)

            rr = Quartz.CGRectIntersection(
                Quartz.CGRectInset(ir, -1.0, -1.0), self._lastBounds
            )

            glScissor(ir.origin.x, ir.origin.y, ir.size.width, ir.size.height)
            glEnable(GL_SCISSOR_TEST)

            glClear(GL_COLOR_BUFFER_BIT)

            if self.respondsToSelector_("drawRect:inCIContext:"):
                self.drawRect_inCIContext_(rr, self._context)

            elif self._image is not None:
                self._context.drawImage_atPoint_fromRect_(self._image, rr.origin, rr)

            glDisable(GL_SCISSOR_TEST)

            # Flush the OpenGL command stream. If the view is double
            # buffered this should be replaced by [[self openGLContext]
            # flushBuffer].

            glFlush()

        else:
            # Printing the view contents. Render using CG, not OpenGL.

            if self.respondsToSelector_("drawRect:inCIContext:"):
                self.drawRect_inCIContext_(ir, self._context)

            elif self._image is not None:
                cgImage = self._context.createCGImage_fromRect_(self._image, ir)

                if cgImage is not None:
                    Quartz.CGContextDrawImage(
                        Cocoa.NSGraphicsContext.currentContext().graphicsPort(),
                        ir,
                        cgImage,
                    )



"""
Python mapping for the ImageIO framework.

This module does not contain docstrings for the wrapped code, check Apple's
documentation for details on how to use these functions and classes.
"""
import sys

import objc
import Quartz.CoreGraphics
from Quartz.ImageIO import _metadata

sys.modules["Quartz.ImageIO"] = mod = objc.ObjCLazyModule(
    "Quartz.ImageIO",
    "com.apple.ImageIO.framework",
    objc.pathForFramework(
        "/System/Library/Frameworks/ApplicationServices.framework/Frameworks/ImageIO.framework"
    ),
    _metadata.__dict__,
    None,
    {
        "__doc__": __doc__,
        "__path__": __path__,
        "__loader__": globals().get("__loader__", None),
        "objc": objc,
    },
    (Quartz.CoreGraphics,),
)


del sys.modules["Quartz.ImageIO._metadata"]



"""
Python mapping for the QuartzCore framework.

This module does not contain docstrings for the wrapped code, check Apple's
documentation for details on how to use these functions and classes.
"""
import sys

import Foundation
import objc
from Quartz.QuartzCore import _metadata
from Quartz.QuartzCore import _quartzcore


# XXX: addConvenienceFor... should be metadata
def CIVector__getitem__(self, idx):
    if isinstance(idx, slice):
        start, stop, step = idx.indices(self.count())
        return [self[i] for i in range(start, stop, step)]

    if idx < 0:
        new = self.count() + idx
        if new < 0:
            raise IndexError(idx)
        idx = new

    return self.valueAtIndex_(idx)


objc.addConvenienceForClass(
    "CIVector",
    (("__len__", lambda self: self.count()), ("__getitem__", CIVector__getitem__)),
)


objc.addConvenienceForClass(
    "CIContext",
    (
        ("__getitem__", lambda self, key: self.objectForKey_(key)),
        ("__setitem__", lambda self, key, value: self.setObject_forKey_(value, key)),
    ),
)
objc.addConvenienceForClass(
    "CIContextImpl",
    (
        ("__getitem__", lambda self, key: self.objectForKey_(key)),
        ("__setitem__", lambda self, key, value: self.setObject_forKey_(value, key)),
    ),
)

objc.addConvenienceForBasicSequence("QCStructure", True)


sys.modules["Quartz.QuartzCore"] = mod = objc.ObjCLazyModule(
    "Quartz.QuartzCore",
    "com.apple.QuartzCore",
    objc.pathForFramework("/System/Library/Frameworks/QuartzCore.framework"),
    _metadata.__dict__,
    None,
    {
        "__doc__": __doc__,
        "__path__": __path__,
        "__loader__": globals().get("__loader__", None),
        "objc": objc,
    },
    (_quartzcore, Foundation),
)


del sys.modules["Quartz.QuartzCore._metadata"]
